/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               DEMO PROGRAMS SET
                          cgroup_test.cpp  -  description
                             -------------------
    begin                : January 3, 2003
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>

#include <sptk/CWindow.h>
#include <sptk/CDataControl.h>
#include <sptk/CGroup.h>
#include <sptk/CButton.h>
#include <sptk/CCheckButtons.h>
#include <sptk/CRadioButtons.h>
#include <sptk/CProgressBar.h>
#include <sptk/CListView.h>
#include <sptk/CComboBox.h>
#include <stdio.h>

void exit_cb(Fl_Widget *w,void *) {
	w->window()->hide();
}

void theme_cb(Fl_Widget *w,void *) {
	try {
		CComboBox *themesCombo = (CComboBox *)w;
		CString themeName = themesCombo->data();

		CThemes::set(themeName);

		CWindow *window = (CWindow *)w->window();
		window->relayout();
		window->redraw();
	}
	catch (CException& e) {
		puts(e.text());
	}
}

int main(int argc,char *argv[]) {

   // CWindow supports auto-layout
	CWindow w(500,400,"CGroup test"); 
	w.resizable(w);
	w.layoutSpacing(4);

	CGroup buttonsGroup("Buttons Group",10,SP_ALIGN_TOP);
	buttonsGroup.frame(FL_THIN_UP_FRAME,FL_THIN_DOWN_FRAME);
	buttonsGroup.align(FL_ALIGN_RIGHT);
	buttonsGroup.layoutSpacing(4);

   // CCheckButtons figures out the size from the list
   // of choices. The default alignment is SP_ALIGN_TOP,
   // and you can change it in ctor
	CCheckButtons cbl("Check Buttons: ");
	cbl.buttons(CStringList("first,second,third,*",","));

	CRadioButtons rbl("Radio Buttons: ");
	rbl.buttons(CStringList("first,second,third,*",","));

	buttonsGroup.end();

	CProgressBar progressBar("Progress: ");
	progressBar.data(50);

   // CListView is more flexible, than CCheckButtons or
   // CRadioButtons, it's vertical size is defined with
   // the layoutSize parameter in ctor as 150.
   // SP_ALIGN_CLIENT allows that widget to occupy all
   // the space left after all the other widgets are 
   // put in place
	CListView listView("List View:",10,SP_ALIGN_CLIENT);
	listView.columns().add(CColumn("column 1",VAR_INT,70));
	listView.columns().add(CColumn("column 2",VAR_INT,70));
	listView.columns().add(CColumn("column 3",VAR_STRING,200));
	listView.columns().add(CColumn("column 4",VAR_STRING));

   // Add 10 items
	char buffer1[10];
	char buffer2[10];
	for(int a=0; a<10; a++) {
		sprintf(buffer1,"%i",a);
		sprintf(buffer2,"%i",100000-a);
		cpchar rowData[] = {buffer1,buffer2, "Column 2", "-----------Long column-----------"};
		CPackedStrings *ps = new CPackedStrings(4,rowData);
		listView.addRow(ps);
	}

   // That group keeps togeteher the buttons. These
   // buttons use the default alignment for buttons - 
   // SP_ALIGN_RIGHT, and the text/icon defined by the 
   // button kind.
	CGroup buttonGroup("",10,SP_ALIGN_BOTTOM);
	buttonGroup.color(FL_LIGHT1);

	CButton okButton(SP_EXIT_BUTTON);
	okButton.callback(exit_cb);

	CComboBox themesCombo("Theme",200,SP_ALIGN_LEFT);
	CStringList themes = CThemes::availableThemes();
	themesCombo.addRows("Theme",themes);
	themesCombo.callback(theme_cb);
	themesCombo.data("Default");

	buttonGroup.end();

	w.end();
	w.show(argc,argv);

	Fl::run();

	return 0;
}
