/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               DEMO PROGRAMS SET
                          odbc_test.cpp  -  description
                             -------------------
    begin                : January 3, 2003
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include <stdio.h>
#include <stdlib.h>

#include <sptk/CODBCDatabase.h>
#include <sptk/CException.h>
#include <sptk/CString.h>
#include <sptk/CDatabase.h>
#include <sptk/CQuery.h>

int testTransactions(CDatabase&db,bool rollback) {
   try {
      CQuery  step5Query(&db,"DELETE FROM test1");
      CQuery  step6Query(&db,"SELECT count(*) FROM test1");

      printf("\n        Begining the transaction ..");
      db.beginTransaction();
      printf("\n        Deleting everything from the temp table ..");
      step5Query.exec();

      step6Query.open();
      int counter = step6Query[0].asInteger();
      step6Query.close();
      printf("\n        The temp table now has %i records ..",counter);

      if (rollback) {
         printf("\n        Rolling back the transaction ..");
         db.rollbackTransaction();
      } else {
         printf("\n        Commiting the transaction ..");
         db.commitTransaction();
      }
      step6Query.open();
      counter = step6Query[0].asInteger();
      step6Query.close();
      printf("\n        The temp table now has %i records..",counter);
   }
   catch (CException& e) {
      puts("Error: " + e.text());
   }

   return true;
}

int main() {

   // If you want to test the database abilities of the data controls
   // you have to setup the ODBC database connection.
   // Typical connect string is something like: "DSN=odbc_demo;UID=user;PWD=password".
   // If UID or PWD are omitted they are read from the datasource settings.
   CODBCDatabase     db("DSN=odbc_demo");

    // Defining the queries
   CQuery  step1Query(&db,"CREATE TEMP TABLE test1 ( id INT, name CHAR(20) )");
   CQuery  step2Query(&db,"INSERT INTO test1 VALUES ( :person_id, :person_name )");
   CQuery  step3Query(&db,"SELECT * FROM test1 WHERE id > :some_id");
   CQuery  step8Query(&db,"DROP TABLE test1");

   try {
      printf("Openning the database.. ");
      db.open();

      printf("Ok.\nStep 1: Creating the temp table.. ");
      step1Query.exec();

      printf("Ok.\nStep 2: Inserting data into the temp table.. ");
        // The following example shows how to use the paramaters
      step2Query.param("person_id") = 1;
      step2Query.param("person_name") = "John Doe";
      step2Query.exec();

      step2Query.param("person_id") = 2;
      step2Query.param("person_name") = "Jane Doe";
      step2Query.exec();

      step2Query.param("person_id") = 3;
      step2Query.param("person_name") = "Anonymous";
      step2Query.exec();

      step2Query.param("person_id") = 4;
      step2Query.param("person_name").setNull();
      step2Query.exec();

      printf("Ok.\nStep 3: Selecting the information the slow way ..\n");
      step3Query.param("some_id") = 1;
      step3Query.open();

      while ( ! step3Query.eof() ) {

         // getting data from the query by the field name
         int id = step3Query["id"].asInteger();

         // another method - getting data by the column number
         CString name = step3Query["name"].asString();

         printf ("\t%i\t%s\n", id, name.c_str() );

         step3Query.fetch();
      }
      step3Query.close();

      printf("Ok.\nStep 4: Selecting the information the fast way ..\n");
      step3Query.param("some_id") = 1;
      step3Query.open();

        // First, find the field references by name or by number
      CField& idField = step3Query[0];
      CField& nameField = step3Query["name"];

      while ( ! step3Query.eof() ) {

            // getting data from the query by the field name
         int id = idField.asInteger();

            // another method - getting data by the column number
         CString name = nameField.asString();

         printf ("\t%i\t%s\n", id, name.c_str() );

         step3Query.fetch();
      }
      step3Query.close();

      printf("Ok.\n***********************************************\nTesting the transactions.");

      testTransactions(db,true);
      testTransactions(db,false);

      printf("Ok.\nStep 6: Closing the database.. ");
      db.close();
      printf("Ok.\n");
   }
   catch (CException& e) {
      printf("\nError: %s\n",e.text().c_str());
      puts("\nSorry, you have to fix your database connection.");
      puts("Please, read the README.txt for more information.");
   }

   return 0;
}
