/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               DEMO PROGRAMS SET
                          registry.cpp  -  description
                             -------------------
    begin                : January 3, 2003
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include <stdio.h>
#include <stdlib.h>

#include <sptk/CRegistry.h>
#include <sptk/CException.h>

void printRegistry(CRegistryMode mode) {
   // Open user settings, file is located in user home directory
   CRegistry   mySettings("mySettings.ini",mode);
   try {
      mySettings.load();
      printf("---> Reading %s\n",mySettings.fileName().c_str());

      mySettings.topic("Window");
      printf("Window position: %s, %s\n",mySettings["x-position"].c_str(),mySettings["y-position"].c_str());

      mySettings.topic("Colors");
      printf("Window header: %s\n",mySettings["header"].c_str());
      printf("Window background: %s\n",mySettings["background"].c_str());
   }
   catch (CException& e) {
      puts(e.text().c_str());
        // Registry file not found, trying to create an empty one.
      FILE *f = fopen(mySettings.fileName().c_str(),"wb");
      if (f) {
         fclose(f);
         puts("Empty registry file is created.");
      }
      else  throw CException("Can't create registry file "+mySettings.fileName());
   }
}

void updateRegistry(CRegistryMode mode) {
   // Open user settings, file is located in user home directory
   CRegistry   mySettings("mySettings.ini",mode);
   try {
      printf("<--- Updating %s\n",mySettings.fileName().c_str());

      mySettings.topic("Window");
      mySettings["x-position"] = 100;
      mySettings["y-position"] = 150;

      mySettings.topic("Colors");
      mySettings["header"] = "RED";
      mySettings["background"] = "0xC0D0FF";
      mySettings.save();
   }
   catch (CException& e) {
      puts(e.text().c_str());
   }
}

int main() {

   puts("-------- Test for  USER  registry. -----------\n");

    // Print the original registry
    // The registry file is located in the user home directory
   printRegistry(USER_REGISTRY);

   updateRegistry(USER_REGISTRY);

    // Print the changed registry
   printRegistry(USER_REGISTRY);

   puts("\n-------- Test for PROGRAM registry. ----------\n");

    // Print the original registry
    // The registry file is located in the current directory
   printRegistry(PROGRAM_REGISTRY);

   updateRegistry(PROGRAM_REGISTRY);

    // Print the changed registry
   printRegistry(PROGRAM_REGISTRY);

   return 0;
}
//---------------------------------------------------------------------------

