/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          string_map.cpp  -  description
                             -------------------
    begin                : 13 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include <stdio.h>
#include <sptk/CException.h>
#include <sptk/CString.h>
#include <sptk/CStringMap.h>

class CTestData {
public:
	CString m_id;
	CString m_data;
	static int objectCounter; 
public:
   // Regular constructor
	CTestData(CString id,CString data) {
		m_id = id;
		m_data = data;
		objectCounter++;
	}

   // Copy constructor
	CTestData(CTestData& td) {
		m_id = td.m_id;
		m_data = td.m_data;
		objectCounter++;
	}

	~CTestData() {
		objectCounter--;
	}

	void print() {
		printf("%20s, %s\n",m_id.c_str(),m_data.c_str());
	}
};

int CTestData::objectCounter;

int main( int argc, char *argv[] ) {
	puts("\n-------------------------\nFilling map.\n");
	try {
		CStringMap<CTestData> map;

		map.insert("Alex",new CTestData("Alex","Programmer"));
		map.insert("Abraam",new CTestData("Abraam","Tester"));
		map.insert("Eric",new CTestData("Eric","Web master"));
		map.insert("Kon",new CTestData("Kon","Manager"));
		map.insert("Arto",new CTestData("Arto","Adviser"));
		map.insert("Aaron",new CTestData("Aaron","Pilot"));
		map.insert("Alex",new CTestData("Alex","DBA"));

		puts("\n-------------------------\nPrinting map.\n");
      // Printing the test data
		CStringMapIterator<CTestData> itor(map);
		for (; *itor; itor++) {
			CTestData *ptr = *itor;
			ptr->print();
		}

		puts("\n-------------------------\nTesting element access.\n");
		CTestData& dt1 = map["Kon"];
		dt1.print();

		CTestData& dt2 = map["Alex"];
		dt2.print();

		puts("\n-------------------------\nTesting element removal.\n");
		printf("Before removal, map includes %i items\n",map.count());
		puts("Removing items: Kon, Alex");
		map.remove("Kon");
		map.remove("Alex");
		printf("After removal, map includes %i items\n",map.count());

		puts("\n-------------------------\nPrinting map.\n");
      // Printing the test data
		itor.first();
		for (; *itor; itor++) {
			CTestData *ptr = *itor;
			ptr->print();
		}

	}
	catch (CException& e) {
		puts(e.text().c_str());
	}

	if (CTestData::objectCounter > 0)
		puts("\nThe number of memory leaks is "+CString(CTestData::objectCounter));

	if (CTestData::objectCounter < 0)
		puts("\nThe number of double-freed objects is "+CString(-CTestData::objectCounter));

	if (CTestData::objectCounter == 0)
		puts("\nAll the objects released correctly\n");

	puts("\n-------------------------\nTest completed.");

	return 0;
}
