/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBase64.cpp  -  description
                             -------------------
    begin                : July 19 2003
    copyright            : (C) 2003 by Alexey Parshin
    email                : alexeyp@gmail.com
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CBASE64_H__
#define __CBASE64_H__

#include <sptk/CStringList.h>
#include <sptk/CBuffer.h>

/// Base64 encoding/decoding
class CBase64 {
public:
   CBase64() { }   /** Default constructor */
   ~CBase64() { }  /** Default destructor */

   /**
    * encode() method encodes (base64) given buffer bufSource
    * to given destination buffer bufDest.
    *
    * @param bufDest CBuffer Destination buffer
    * @param bufSource CBuffer Source buffer
    * @see encode(CString& strDest, const CBuffer& bufSource)
    */
   static void encode(CBuffer& bufDest, const CBuffer& bufSource);

   /**
    * This encode() method encodes (base64) given buffer bufSource
    * and returns CString object.
    *
    * @param strDest CString Destination string
    * @param bufSource CBuffer* Source buffer  
    * @see encode(CBuffer& bufDest, const CBuffer& bufSource)
    */
   static void encode(CString& strDest, const CBuffer& bufSource);

   /** 
    * Decodes base64 encoded buffer "string" into buffer "bufDest" 
    * @param bufDest CBuffer destination buffer
    * @param bufSource CBuffer source buffer that holds base64 decoded data
    * @returns length of returned buffer, or -1 on error
    */
   static int decode(CBuffer &bufDest, const CBuffer &bufSource);

   /** 
    * Decodes base64 encoded string "sArg" into buffer "bufDest"
    * @param bufDest CBuffer destination buffer
    * @param strSource CString source string that holds base64 decoded data
    * @returns length of returned buffer, or -1 on error
    */
   static int decode(CBuffer &bufDest, const CString &strSource);
};

#endif
