/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFTPConnect.h  -  description
                             -------------------
    begin                : July 19 2003
    copyright            : (C) 2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CString.h>
#include <sptk/CStringList.h>
#include <sptk/CSocket.h>

/// FTP socket connects to FTP server. It takes two of such sockets
/// to establish the correct FTP communication
class CFTPSocket : public CSocket {
   /// Server response string list
   CStringList  m_response;
public:
   /// Default constructor
   CFTPSocket();

   /// Destructor
   ~CFTPSocket();

   /// Establish the connection
   virtual void open(CString hostName="", int port=0);

   /// Returns a reference to server response string list
   const CStringList& response() const { return m_response; }

   /// Logs in to the server
   const CStringList& login(CString user,CString password);

   /// Sends a command to the server
   const CStringList& command(CString cmd);

   /// Reads server response after calling a command
   const CStringList& get_response();
};

/// Connection to the FTP server
class CFTPConnect {
protected:

   /// FTP command socket
   CFTPSocket        m_commandSocket;
   /// FTP data socket
   CFTPSocket        m_dataSocket;
   /// FTP user name
   CString           m_user;
   /// FTP user password
   CString           m_password;
   /// FTP host name
   CString           m_host;
   /// FTP port
   int               m_port;
   /// Passive mode
   bool              m_passive;

protected:

   /// Opens data port on the FTP server
   void openDataPort();

   /// Sends command to the FTP server
   void command(CString cmd);

   /// Gets the list from the FTP server
   void getList(CString cmd,CStringList& list);

public:

   /// Constructor
   CFTPConnect();

   /// Destructor
   ~CFTPConnect();

   /// Sets the passive mode true/false
   void passive(bool p) { m_passive = p; }

   /// Returns the passive mode
   bool passive() const { return m_passive; }

   /// Sets the user name
   void user(CString u) { m_user = u; }

   /// Returns the user name
   CString user() const { return m_user; }

   /// Sets the user password
   void password(CString p) { m_password = p; }

   /// Returns the user password
   CString password() const { return m_password; }

   /// Sets the host name and port
   void host(CString hostName,int portNumber=21);

   /// Opens the FTP connection
   void open();

   /// Closes the FTP connection
   void close();

   /// Returns the referense on the server response string list
   const CStringList& response() const { return m_commandSocket.response(); }

   /// Returns true if the connection is active
   bool active() const { return m_commandSocket.active(); }

   /// FTP quit command
   void cmd_quit();

   /// FTP type command
   void cmd_type(char type);

   /// FTP cd command
   void cmd_cd(CString dir);

   /// FTP pwd command
   void cmd_pwd();

   /// FTP list command
   void cmd_list(CStringList& result);

   /// FTP nlist command
   void cmd_nlst(CStringList& result);

   /// FTP retr command
   void cmd_retr(CString fileName);

   /// FTP store command
   void cmd_store(CString fileName);
};
