/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CListViewSelection.h  -  description
                             -------------------
    begin                : Jul 14 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CLISTVIEWSELECTION_H__
#define __CLISTVIEWSELECTION_H__

#include <sptk/CPackedStrings.h>

#define CLV_SELECTED       1
#define CLV_CHECKED        2
#define CLV_NOTDISPLAYED   4

/// The selection list for CListView. Allows CListView to work with one or few selected rows
/// without scanning through all the rows in list view for selected rows.
class SP_EXPORT CSelection {

   /// The list of the rows
   CList          m_selectedRows;

public:

   /// Default constructor
   CSelection()  {}

   /// Destructor
   ~CSelection() { clear(); }

   /// Selects a row and adds it to the selection list
   void select(CPackedStrings *row);

   /// Deselects a row and removes it from the selection list
   void deselect(CPackedStrings *row);

   /// Removes the row from the selection list
   void remove(CPackedStrings *row) { m_selectedRows.remove(row); }

   /// Deselects all the rows in the selection list and empties the list
   void deselectAll();

   /// Removes all rows from the selection list
   void clear();

   /// Returns the number of selected rows
   unsigned count() const { return m_selectedRows.count(); }

   /// Element access to the selection, const version only
   /// @param index int, index in the selection list
   CPackedStrings& operator[] (int index) const { return *(CPackedStrings *)m_selectedRows[index]; }

   /// Finds an item with a particular key value. Returns NULL if not found.
   /// @param keyValue int, key value
   CPackedStrings *findKey(int keyValue) const;

   /// Finds an item with a particular caption. Returns NULL if not found.
   /// @param caption CString, item caption (a string in the first column)
   CPackedStrings *findCaption(CString caption) const;
};

#endif
