/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CParams.h  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CPARAMS_H_
#define __CPARAMS_H_

#include <sptk/CStringList.h>
#include <sptk/CVariant.h>
#include <sptk/CIntList.h>

/// @addtogroup Database Database Support
/// @{

/// The SQL query binary parameter. Used to simplify the ODBC parameter binding.
class SP_EXPORT CParam : public CVariant {
   friend class CQuery;
   friend class CParamList;

   CString           m_name;              ///< Parameter name
   CIntList          m_bindParamIndexes;  ///< The list of SQL query parameter numbers with this name

protected:

   char             *m_timeData;          ///< Special memory allocated for time structures
   long              m_callbackLength;    ///< An integer reserved to callback parameter data length

public:   
   /// Clears internal parameter binding index
   void     bindClear() { m_bindParamIndexes.clear(); }

   /// Adds internal parameter binding index
   void     bindAdd(unsigned bindIndex);

   /// Returns internal parameter binding count
   unsigned bindCount() { return m_bindParamIndexes.count(); }

   /// Returns the parameter bing position by index in the binding list
   unsigned bindIndex(unsigned ind) { return m_bindParamIndexes[ind]; }

   /// Returns the internal small conversion buffer used to convert the
   /// date structure to SPTK. Please, don't use it.
   char *conversionBuffer() { return m_timeData; }

   /// An integer reserved to callback parameter data length
   long& callbackLength() { return m_callbackLength; }

public:

   /// Constructor
   /// @param name char *, parameter name
   CParam(char *name);

   /// Destructor
   ~CParam();

   /// Returns parameter name
   CString  name() const { return m_name; }

   /// Assign operator
   CParam& operator = (const CParam& param);

   /// Assign operator
   CParam& operator = (const CVariant& param) { if (this == &param) return *this; setData(param); return *this; };

   /// Assign operator
   CParam& operator = (int v)                 { setInteger(v); return *this; }

   /// Assign operator
   CParam& operator = (unsigned v)            { setInteger(v); return *this; }

   /// Assign operator
   CParam& operator = (float v)               { setFloat(v);   return *this; }

   /// Assign operator
   CParam& operator = (double v)              { setFloat(v);   return *this; }

   /// Assign operator
   CParam& operator = (const char *s)         { setString(s);  return *this; }

   /// Assign operator
   CParam& operator = (const CString& s)      { setString(s.c_str()); return *this; }

   /// Assign operator
   CParam& operator = (const CDateTime& dt)   { setDateTime(dt); return *this; }
};

/// The list of the query parameters. 
/// @see CQuery 
/// @see CParam
class SP_EXPORT CParamList {
   friend class CQuery;

   /// The list of parameters
   CList               m_items;
protected:   
   /// Adds a parameter to the list
   void     add(const CParam * item);

   /// Removes the parameter from the list
   /// @param index unsigned, the parameter index
   void     remove(unsigned index);

public:

   /// Default constructor
   CParamList() {};

   /// Destructor
   ~CParamList();

   /// Returns parameter index by name
   /// @param paramName const char *, parameter name
   /// @returns parameter index, or -1 if not found
   int      paramIndex(const char *paramName);

   /// Parameter access by index
   /// @param index unsigned, parameter index
   CParam&  operator[] (unsigned index) const;

   /// Parameter access by index
   /// @param index int, parameter index
   CParam&  operator[] (int index) const;

   /// Parameter access by name
   /// @param paramName const char *, parameter name
   CParam&  operator[] (const char *paramName) const;

   /// Parameter access by name
   /// @param paramName const CString&, parameter name
   CParam&  operator[] (const CString& paramName) const;

   /// Returns parameter count
   unsigned count() const { return m_items.count(); }
};


/// @}

#endif
