/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRegistry.h  -  description
                             -------------------
    begin                : Tue Mar 26 2002
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef cregistryH
#define cregistryH

#include <sptk/CString.h>
#include <sptk/CStringList.h>
#include <sptk/CStringMap.h>

class CRegistry;
class CRegistryTopic;

/// Class CRegistryItem is a single registry item that contains a name and a value
class SP_EXPORT CRegistryItem {
   friend class CRegistryTopic;
   /// Registry item name
   CString m_name;
public:
   /// Constructor
   /// @param buffer CString, the input data in fomat "name=value"
   CRegistryItem(const CString buffer="");
   /// Copy constructor
   CRegistryItem(const CRegistryItem& item) { m_name = item.m_name; value = item.value; }

   /// Registry item name
   const CString& name()  const { return m_name;  }

   /// Registry item value
   CString value;
};

typedef CStringMap<CRegistryItem> CRegistryItems;

/// Class CRegistryTopic represents a list of CRegistryItem.
class SP_EXPORT CRegistryTopic : public CRegistryItems {
   friend class CRegistry;

   CString m_name;  ///< Registry topic name

public:   
   /// Class Iterator
   typedef CStringMapIterator<CRegistryItem> Iterator;

public:
   /// Constructor
   /// @param name CString, unique topic name
   CRegistryTopic(const CString name) { m_name = name; }
   /// Adds an item to the topic
   /// @param buffer CString, the input data in fomat "name=value"
   void addItem(const CString buffer);
   /// Registry topic name
   const CString& name() const { return m_name; }
};

/// Modes for the registry. User registry is stored in home directory
/// (if home directory is avalable). Program registry is stored in the program's
/// startup directory.
enum CRegistryMode { USER_REGISTRY=1, PROGRAM_REGISTRY=2 };

/// CRegistry is the class that works with INI-files.
class SP_EXPORT CRegistry {
   CString         m_emptyString;  ///< An empty string, internal
   CString         m_fileName;     ///< The registry file name
   CList           m_topics;       ///< The list of topics
   CString         m_topic;        ///< The current topic name
   CRegistryTopic *m_currentTopic; ///< The current topic pointer
public:

   /// Constructor
   /// @param fileName const char *, the registry file name w/o path
   /// @param mode CRegistryMode,
   CRegistry(const char *fileName,CRegistryMode mode=USER_REGISTRY);

   /// Destructor
   virtual  ~CRegistry();

   /// Sets current topic name. If the topic doesn't exist it will be created.
   void     topic(const CString topic);

   /// Returns current topic
   CRegistryTopic& topic() { return *m_currentTopic; }

   /// Returns current topic
   const CRegistryTopic& topic() const { return *m_currentTopic; }

   /// Returns the registry file name
   const CString fileName() { return m_fileName; }

   /// Access to the registry item inside the current topic by item name
   CString& operator[] (const char *name);

   /// Access to the registry item inside the current topic by item name
   const CString& operator[] (const char *name) const ;

   /// Loads registry from the file. Usually it's the first action with registry.
   virtual void load();

   /// Saves registry to the file. 
   virtual void save();

   /// Loads registry from the string list
   virtual void load(CStringList& data);

   /// Saves registry to the the string list
   virtual void save(CStringList& data);

   /// Clears the registry content
   virtual void clear();

public:   

   /// Finds out the user's home directory
   static CString homeDirectory();
};
//---------------------------------------------------------------------------
#endif
