/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CString.h  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CSTRING_H__
#define __CSTRING_H__

#include <sptk/sptk.h>

class CString;
class CVariant;
class CField;
class CBuffer;

bool operator <  (const CString &s1, const CString &s2);
bool operator <  (const CString &s1, const CString &s2);
bool operator <= (const CString &s1, const CString &s2);
bool operator >  (const CString &s1, const CString &s2);
bool operator >= (const CString &s1, const CString &s2);
bool operator == (const CString &s1, const CString &s2);
bool operator != (const CString &s1, const CString &s2);

bool operator <  (const CString &s1, const char *s2);
bool operator <= (const CString &s1, const char *s2);
bool operator >  (const CString &s1, const char *s2);
bool operator >= (const CString &s1, const char *s2);
bool operator == (const CString &s1, const char *s2);
bool operator != (const CString &s1, const char *s2);

/// Class CString represents the general string object.
class SP_EXPORT CString {
   /// Add operator
	friend CString operator +(const char*, const CString& rhs);
protected:
   /// String buffer
	char *str;
   /// Replaces string with another string
	void assign(const char *st);
public:
   /// Constructor
   /// @param c char, a single character to fill the string
   /// @param repeater int, the number of characters to fill the string
	CString(char c,int repeater=1);

   /// Constructor
   /// @param number int, an integer to be converted to the string
	CString(int number);

   /// Constructor
   /// @param number unsigned, an unsigned integer to be converted to the string
	CString(unsigned number);

   /// Default constructor
	CString();

   /// Constructor
   /// @param s const char *, a string
	CString(const char *s);

   /// Constructor
   /// @param s const char *, a string with no more than maxlen to copy into 
   /// @param maxlen int, maximum characters to copy 
	CString(const char *s,int maxlen);

   /// Constructor
   /// @param s const CString&, a string
	CString(const CString& s);

   /// Constructor
   /// @param b const CBuffer&, a buffer that contains a zero-terminated string
	CString(const CBuffer& b);

   /// Destructor
	~CString();
public:
   /// Assignment operation
	CString& operator = (const char *s);
   /// Assignment operation
	CString& operator = (const CString& s);
   /// Assignment operation
	CString& operator = (char ch);
   /// Assignment operation
	CString& operator = (int);
   /// Assignment operation
	CString& operator = (unsigned);
   /// Assignment operation
	CString& operator = (long);
   /// Assignment operation
	CString& operator = (float);
   /// Assignment operation
	CString& operator = (double);
   /// Assignment operation
	CString& operator = (const CVariant& v);
   /// Assignment operation
	CString& operator = (CField& v);
   /// Assignment operation
	CString& operator = (const CBuffer& v);
public:
   /// Conversion operation
	operator const char * () const { return str;              }
   /// Conversion operation
	operator     unsigned () const { return toInt();          }
   /// Conversion operation
	operator          int () const { return toInt();          }
   /// Conversion operation
	operator         long () const { return toInt();          }
   /// Conversion operation
	operator        float () const { return (float)toFloat(); }
   /// Conversion operation
	operator       double () const { return toFloat();        }
public:
   /// Comparision operation
	friend bool operator <  (const CString &s1, const CString &s2);
   /// Comparision operation
	friend bool operator <= (const CString &s1, const CString &s2);
   /// Comparision operation
	friend bool operator >  (const CString &s1, const CString &s2);
   /// Comparision operation
	friend bool operator >= (const CString &s1, const CString &s2);
   /// Comparision operation
	friend bool operator == (const CString &s1, const CString &s2);
   /// Comparision operation
	friend bool operator != (const CString &s1, const CString &s2);

   /// Comparision operation
	friend bool operator <  (const CString &s1, const char *s2);
   /// Comparision operation
	friend bool operator <= (const CString &s1, const char *s2);
   /// Comparision operation
	friend bool operator >  (const CString &s1, const char *s2);
   /// Comparision operation
	friend bool operator >= (const CString &s1, const char *s2);
   /// Comparision operation
	friend bool operator == (const CString &s1, const char *s2);
   /// Comparision operation
	friend bool operator != (const CString &s1, const char *s2);

   /// Comparision operation
	friend bool operator <  (const char *s1, const CString &s2);
   /// Comparision operation
	friend bool operator <= (const char *s1, const CString &s2);
   /// Comparision operation
	friend bool operator >  (const char *s1, const CString &s2);
   /// Comparision operation
	friend bool operator >= (const char *s1, const CString &s2);
   /// Comparision operation
	friend bool operator == (const char *s1, const CString &s2);
   /// Comparision operation
	friend bool operator != (const char *s1, const CString &s2);

public:
   /// Add operation (concatenation)
   /// @returns new CString as a result of concatenation
	CString  operator +  (char * s) const;

   /// Add operation (concatenation)
   /// @returns new CString as a result of concatenation
	CString  operator +  (const char * s) const;

   /// Add operation (concatenation)
   /// @returns new CString as a result of concatenation
	CString  operator +  (const CString& s) const;

   /// Add operation (concatenation)
   /// @returns new CString as a result of concatenation
	CString  operator +  (char ch) const;

   /// Add operation (concatenation)
   /// @returns this object as a result of concatenation
	CString& operator += (const char * s);

   /// Add operation (concatenation)
   /// @returns this object as a result of concatenation
	CString& operator += (const CString& s);

   /// Add operation (concatenation)
   /// @returns this object as a result of concatenation
	CString& operator += (char ch);
public:

   /// Empties the string
	void     clear();

   /// Makes a copy of the string and removes the pattern from it
   /// @param pattern const char *, substring to remove
	CString remove(const char *pattern) const;

   /// Makes a copy of the string and replaces the pattern in it
   /// @param pattern const char *, substring to replace
   /// @param replacement const char *, replacement substring
	CString replace(const char *pattern,const char *replacement) const;

   /// Trims the trailing spaces
	CString  trimRight() const;

   /// Trims the leading spaces
	CString  trimLeft()  const;

   /// Trims the trailing and leading spaces
	CString  trim()      const;

   /// Converts string to lower case
	CString  lowerCase() const;

   /// Converts string to upper case
	CString  upperCase() const;

   /// For every word in the string longer than three characters capitalize the first letter, and make the rest of the word lowercase.
	CString  capitalizeWords() const;

   /// Returns substring of the string
   /// @param start int, the first character position
   /// @param count int, the max character count in substring
	CString  subString(int start,int count) const;

   /// Removes substring from the string
   /// @param start int, the first character position
   /// @param count int, the max character in substring to remove
	void     subDelete(int start,int count=1);

   /// Removes substring from the string
   /// @param s const char *, the substring to remove
	void     subDelete(const char *s) { subReplace(s,0L); }

   /// Inserts substring into the string
   /// @param start int, the insert position
   /// @param str const char *, the substring to insert
	void     subInsert(int start,const char *str);

   /// Inserts substring into the string
   /// @param start int, the insert position
   /// @param ch char, the character to insert
	void     subInsert(int start,char ch);

   /// Replaces substring in the string
   /// @param s_str const char *, the string to find and replace
   /// @param r_str const char *, the replacement string
	void     subReplace(const char *s_str,const char *r_str);

   /// Returns a word at position
   /// @param position long, character in word position
	CString  wordAt(long position) const;

   /// Splits string on two parts on the wrap character near limit, and copies the second part into remainder
   /// @param limit long, the wrap length
   /// @param remainder CString *, the string to store the remainder
	CString& wrapAt(long limit, CString *remainder=0L);

   /// Splits string on two parts, and copies the second part into remainder
   /// @param limit long, the split length
   /// @param remainder CString *, the string to store the remainder
	CString& chopAt(long limit, CString *remainder=0L);

   /// Returns the string length
	unsigned length() const;

   /// Finds the sub string in the string
   /// @returns the sub string's position, or -1 if not found
	int      pos(const char *substr) const;

   /// Returns the result of conversion to integer number
   /// @param defvalue int, the value to return if conversion wasn't successful
	int      toInt(int defvalue=0) const;

   /// Returns the result of conversion to floating point number
   /// @param defvalue float, the value to return if conversion wasn't successful
	double   toFloat(float defvalue=0) const;

   /// Returns const char * to the internal string buffer
	const char *c_str() const { return str; }

   /// Returns true if the string has zero length
	bool     isEmpty() const;

   /// Sets the new string length
	void     setLength(int newLength);

   /// Substitutes the decimal character codes in the string with the characters.
   /// The character code should have certain prefix
   /// @param prefix char, the prefix character
	CString  substituteSpecialChars(char prefix) const;

public:

   /// Character access
	char& operator [] (int i)            { return str[i]; }
   /// Character access
	char  operator [] (int i) const      { return str[i]; }
   /// Character access
	char& operator [] (unsigned i)       { return str[i]; }
   /// Character access
	char  operator [] (unsigned i) const { return str[i]; }
};

#endif
