/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWaiter.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CWAITER_H__
#define __CWAITER_H__

#ifndef _WIN32

#include <unistd.h>
#include <pthread.h>

#else

#include <windows.h>
#include <process.h>

#endif

#include <sptk/sptk.h>

/// The synchronization object
class SP_EXPORT CWaiter {
   /// Is signal sent?
   bool              m_signalSent;
   /// Are we waiting for the signal?
   int               m_waitingForSignal;

protected:

#ifndef _WIN32
   /// Mutex object
   pthread_mutex_t   m_waiter;
   /// Condition object
   pthread_cond_t    m_condition;
#else
   /// Event object
   HANDLE            m_waiter;
#endif

public:

   /// Constructor
   CWaiter();

   /// Destructor
   ~CWaiter();

   /// Try to lock waiter. Blocks until the lock.
   void lock();

   /// Unlocks the waiter.
   void unlock();

   /// Waits for a signal, timeout in msec
   /// @param timeout int, timeout in milliseconds 
   int  waitForSignal(int timeout=-1);   

   /// Sends signal to stop waiting
   void sendSignal();                    
};


#endif
