/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXml.h  -  description
                             -------------------
    begin                : Sun May 22 2003
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CXML_H__
#define __CXML_H__

#include <sptk/CStringMap.h>

#include <sptk/xml/CXmlDoc.h>
#include <sptk/xml/CXmlNode.h>
#include <sptk/xml/CXmlParser.h>
#include <sptk/xml/CXmlTokenizer.h>

/// @addtogroup XML
/// @{

/// XML extension error codes enumeration.
enum ErrorCodes { 
   XML_NO_ERROR = 0,       /**< Operation success */
   XML_UNKNOWN,            /**< Unspecified or unknown error */
   XML_OPENTAG_EXPECTED,      /**< Expected an open tag literal '<' */
   XML_OPENTAG_CDATA_EXPECTED, /**< Expected a '<' or cdata */
   XML_CLOSETAG_EXPECTED,     /**< Expected a '>' closing tag literal */
   XML_PI_DOCTYPE_EXPECTED,   /**< Expected a processing instruction or doctype tag */
   XML_TAGNAME_EXPECTED,      /**< Expected a tag name after '<' or '</' */
   XML_COMMENT_START_EXPECTED,   /**< Starting '--' of comment expected after '!' */
   XML_COMMENT_END_MISSING,   /**< Ending '-->' missing */
   XML_SLASH_EXPECTED,        /**< Expected a '/' after closing tag literal '<' */
   XML_TAGNAME_MISMATCH,      /**< Tag name from start and end tag mismatch */
   XML_EQUAL_EXPECTED,        /**< Expected '=' after attribute name */
   XML_VALUE_EXPECTED         /**< Expected value after an '=' in attribute */
};


/**
 * Xml extension throws CXmlException type exceptions.
 * You should catch always atleast these type of exceptions,
 * when processing XML.
 */
class SP_EXPORT CXmlException
{
public:

   /**
    * Constructs new exception
    * @param error int, error code for error
    * @param locator CXmlLocator *, XML locator for stream
    */
   CXmlException(int error, CXmlLocator *locator) : m_locator(locator), m_error(error) { }

   /**
    * Returns human readable error string.
    * @param short_version bool, return short version of error string
    */
   CString text(bool short_version=false) const;

    /** 
    * Returns errorcode of last error happened.
    */
   int error() const { return m_error; }

   /**
    * Returns locator for error occured.
    * NULL, if no locator specified in CXmlParser
    */
   CXmlLocator *locator() const { return m_locator; }

   /**
    * Return error string for given errorcode.
    */
   static const char *error_string(int error);

private:
   CXmlLocator *m_locator;
   int m_error;
};

/** @} */

#endif
