/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlNode.h  -  description
                             -------------------
    begin                : Sun May 22 2003
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CXML_NODELIST_H__
#define __CXML_NODELIST_H__

#include <sptk/CList.h>

class CXmlNode;

/// @addtogroup XML
/// @{

/**
 * The CXmlNodeList interface provides the an ordered collection of nodes, 
 * The items in the NodeList are accessible via an integral index, starting from 0.
 */
class CXmlNodeList : public CList {
public:
   /// Constructor
   CXmlNodeList() { }

   /// Item access
   /// @param index unsigned, item index
   /// @returns item pointer
   CXmlNode *item(unsigned index) const { return (CXmlNode*)m_items[index]; }

   /// Item access
   /// @param index unsigned, item index
   /// @returns item pointer
   CXmlNode *operator [](unsigned index) const { return (CXmlNode *)m_items[index]; }

   /// Appends an item after the last item
   /// @param item CXmlNode *, item to append
   void append(CXmlNode *item) { CList::add(item); }

   /// Inserts an item before the first item
   /// @param item CXmlNode *, item to insert
   void prepend(CXmlNode *item) { CList::insert(0,item); }

   /// Inserts an item
   /// @param pos unsigned, insert position
   /// @param item CXmlNode *, item to insert
   void insert(unsigned pos, CXmlNode *item) { CList::insert(pos, item); }

   /// Replaces the item
   /// @param pos unsigned, replace position
   /// @param item CXmlNode *, item to insert
   void replace(unsigned pos, CXmlNode *item) { m_items[pos] = item; }

   /// Removes the item
   /// @param pos unsigned, remove item position
   void remove(unsigned pos) { CList::remove(pos); }

   /// Removes the item
   /// @param item CXmlNode *, item to remove
   bool remove(CXmlNode *item) { return CList::remove(item); }

   /// Returns the item index in the list
   /// @param item CXmlNode *, item to find
   /// @returns item index, or -1 if not found
   int indexOf(const CXmlNode *item) const { return CList::indexOf(item); }

   /// Internal pointer to the items' list
   CXmlNode **data() { return (CXmlNode**)m_items; }

protected:
   /// Releases item's memory
   void free_item(CXmlNode *item);
};

/// @}

#endif
