/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBaseMailConnect.cpp  -  description
                             -------------------
    begin                : 25 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <stdio.h>
#include <string.h>

#include <sptk/CBase64.h>
#include <sptk/CBaseMailConnect.h>
#include <sptk/CStringMap.h>
#include <sptk/CDateTime.h>
#include <time.h>

#define LINE_CHARS 72

class CContentTypes {
	static CStringMap<CString> m_contentTypes;
public:
	CContentTypes() { if (!m_contentTypes.count()) init(); }
	void init();
	static CString type(CString fileName);
};

CStringMap<CString> CContentTypes::m_contentTypes(false);
static CContentTypes contentTypes;
static const CString boundary1(  "==_MIME_BOUNDARY_=="    );
static const CString boundary2("--==_MIME_BOUNDARY_==\n\r");
static const CString boundary3("\n\r--==_MIME_BOUNDARY_==--\n\r");

void CContentTypes::init() {
	m_contentTypes.insert("txt", "text/plain");
	m_contentTypes.insert("htm", "text/html");
	m_contentTypes.insert("html","text/html");
	m_contentTypes.insert("gif", "image/gif");
	m_contentTypes.insert("png", "image/png");
	m_contentTypes.insert("bmp", "image/bmp");
	m_contentTypes.insert("jpg", "image/jpeg");
	m_contentTypes.insert("tif", "image/tiff");
	m_contentTypes.insert("pdf", "application/pdf");
	m_contentTypes.insert("xls", "application/vnd.ms-excel");
	m_contentTypes.insert("csv", "text/plain");
	m_contentTypes.insert("doc", "application/msword");
	m_contentTypes.insert("wav", "application/data");
}

CString CContentTypes::type(CString fileName) {
	const char *extension = strrchr(fileName.c_str(),'.');
	if (!extension)
		return "application/octet-stream";
	extension++;
	if (strlen(extension) > 4)
		return "application/octet-stream";
	const CString *contentType = m_contentTypes.find(extension);
	if (contentType) return *contentType;
	else
		return "application/octet-stream";
}

void CBaseMailConnect::mimeFile(CString fileName,CString fileAlias,CBuffer& buffer) {
	CBuffer  bufSource;
	CString  strDest;
	char    *header = new char[1024];

	bufSource.loadFromFile(fileName);

	CString ctype = contentTypes.type(fileName.trim());

	sprintf(header,"Content-Type: %s; name=\"%s\"\n\r",ctype.c_str(),fileAlias.c_str());
	strcat(header,"Content-Transfer-Encoding: base64\n\r\n\r");

	buffer.append(header);

	CBase64::encode(strDest,bufSource);
	unsigned cnt = strDest.length();
	const char *data = strDest.c_str();
	char line[90];
	for (unsigned p = 0; p < cnt; p += LINE_CHARS) {
		unsigned length = cnt - p;
		if (length > LINE_CHARS) length = LINE_CHARS;
		memcpy(line,data+p,80);
		line[length] = '\n';
		line[length+1] = '\r';
		line[length+2] = 0;
		buffer.append(line,length+2);
	}
}

void CBaseMailConnect::mimeMessage(CBuffer& buffer) {
	CString headers;

	if (m_from.length())
		headers += "From: " + m_from + "\n\r";
	else  headers += "From: postmaster\n\r";

	m_to.subReplace(";",", ");
	headers += "To: " + m_to + "\n\r";

	if (m_cc.length()) {
		m_cc.subReplace(";",", ");
		headers += "CC: " + m_cc + "\n\r";
	}

	headers += "Subject: " + m_subject + "\n\r";

	CDateTime   date = CDateTime::Now();
	short dy,dm,dd,th,tm,ts,tms;
	date.decodeDate(&dy,&dm,&dd);
	date.decodeTime(&th,&tm,&ts,&tms);

	char  dateBuffer[128];
	char  *sign = "-";
	int   offset = CDateTime::timeZoneOffset;
	if (offset >= 0) 
		sign = "";
	else
		offset = -offset;

	sprintf(dateBuffer,
		"Date: %s, %i %s %04i %02i:%02i:%02i %s%04i (%s)\n\r",
		date.dayOfWeekName().subString(0,3).c_str(),
		dd,
		CDateTime::monthNames[dm-1].subString(0,3).c_str(),
		dy,
		th,tm,ts,
		sign,
		offset * 100,
		CDateTime::timeZoneName.c_str() 
	);


	headers += dateBuffer;

	CString mimeHeaders("MIME-Version: 1.0\n\r");

	if (m_body.type() == MMT_PLAIN_TEXT_MESSAGE) {

		mimeHeaders += "Content-Type: multipart/mixed;\n\r boundary=\"" + boundary1 + "\"\n\r";
		mimeHeaders += "\n\rThis message is in MIME format.";
		mimeHeaders += " Since your mail reader does not understand this format, some or all of this message may not be legible.\n\r\n\r";

		mimeHeaders += boundary2;
		mimeHeaders += "Content-Type: text/plain; charset=\"us-ascii\"\n\r";
		mimeHeaders += "Content-Transfer-Encoding: 7bit\n\r";
		mimeHeaders += "\n\r" + m_body.text() + "\n\r\n\r";

	} else {
/*
		mimeHeaders += "Content-Type: multipart/alternative;\n\r boundary=\"" + boundary1 + "\"\n\r";
		mimeHeaders += "\n\rThis message is in MIME format.";
		mimeHeaders += " Since your mail reader does not understand this format, some or all of this message may not be legible.\n\r\n\r";

		mimeHeaders += boundary2;
		mimeHeaders += "Content-Type: text/plain; charset=\"us-ascii\"\n\r";
		mimeHeaders += "Content-Transfer-Encoding: 7bit\n\r";
		mimeHeaders += "\n\r" + m_body.text() + "\n\r\n\r";

		mimeHeaders += boundary2;
		mimeHeaders += "Content-Type: text/html; charset=\"us-ascii\"\n\r";
		mimeHeaders += "Content-Transfer-Encoding: 7bit\n\r";
		mimeHeaders += "\n\r" + m_body.html() + "\n\r\n\r";
*/
		mimeHeaders += "Content-Type: multipart/mixed;\n\r boundary=\"" + boundary1 + "\"\n\r";
		mimeHeaders += "\n\rThis message is in MIME format.";
		mimeHeaders += " Since your mail reader does not understand this format, some or all of this message may not be legible.\n\r\n\r";

		mimeHeaders += boundary2;
		mimeHeaders += "Content-Type: text/html; charset=\"us-ascii\"\n\r";
		mimeHeaders += "Content-Transfer-Encoding: 7bit\n\r";
		mimeHeaders += "\n\r" + m_body.html() + "\n\r\n\r";
	}

	buffer.reset(2048);
	buffer.append(headers,headers.length());
	buffer.append(mimeHeaders,mimeHeaders.length());

	CStringList sl(m_attachments,';');
	for (unsigned i = 0; i < sl.count(); i++) {
		CString attachment = sl[i];
		CString attachmentAlias = attachment;
		char separator = '\\';
		if (attachment.pos("/") >= 0)
			separator = '/';
		CStringList attachmentParts(attachment,separator);
		int attachmentPartsCount = attachmentParts.count();
		if (attachmentPartsCount > 1)
			attachmentAlias = attachmentParts[attachmentPartsCount-1];
		if (attachment.length()) {
			buffer.append(boundary2,boundary2.length());
			mimeFile(attachment,attachmentAlias,buffer);
		}
	}
	buffer.append(boundary3,boundary3.length());
}
