/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CControlList.h  -  description
                             -------------------
    begin                : 12 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <FL/Fl_Group.H>

#include <string.h>
#include <stdlib.h>
#include <sptk/CException.h>
#include <sptk/CControlList.h>

typedef CStringMapIterator<CControl *> CStringControlIterator;

CControlList::CControlList()
: CStringControlMap(true) { 
}

void CControlList::scanControls(const Fl_Group *group) {
   unsigned cnt = group->children();
   for (unsigned i = 0; i < cnt; i++) {
      Fl_Widget *widget = group->child(i);
      
      // The try {} catch() {} is only required for MSVC++
      CControl *control = 0L;
      try {
         control = dynamic_cast<CControl *>(widget);
      }
      catch (...) {
      }

      if (control) {
         if (control->fieldName().length())
            add(control);
         continue;
      }

      Fl_Group *g = 0L;
      
      try {
         g = dynamic_cast<Fl_Group *>(widget);
      }
      catch (...) {
      }

      if (g) 
         scanControls(g);
   }
}

void CControlList::add(CControl *control) {
   insert(control->fieldName(),control);
}

void CControlList::add(const Fl_Group& group) {
   scanControls(&group);
}

void CControlList::add(const CControlList& list) {
   CStringControlIterator itor(list);
   for ( ; *itor; itor++)
      add(*(*itor));
}

void CControlList::remove(const CControlList& l) {
   CStringControlIterator itor(l);
   for (;*itor;itor++)
      remove(*(*itor));
}

const CControl & CControlList::operator [] (unsigned ndx) const {
   CControl *c = item(ndx);
   return *c;
}

CControl & CControlList::operator [] (unsigned ndx) {
   CControl *c = item(ndx);
   return *c;
}

const CControl & CControlList::operator [] (const CString& fieldName) const {
   return *CStringControlMap::operator [] (fieldName.c_str());
}

CControl & CControlList::operator [] (const CString& fieldName) {
   return *CStringControlMap::operator [] (fieldName.c_str());
}

const CControl & CControlList::operator [] (const char *fieldName) const {
   CControl **control = find(fieldName);
   if (control)
      return *(*control);
   throw CException("Field "+CString(fieldName)+" not found.");
}

CControl & CControlList::operator [] (const char *fieldName) {
   CControl **control = find(fieldName);
   if (control) return *(*control);
   throw CException("Field "+CString(fieldName)+" not found.");
}

void CControlList::reset() {
   CStringControlIterator itor(*this);
   for ( ; *itor; itor++ ) {
      (*(*itor))->reset();
   }
}
