/***************************************************************************
            SIMPLY POWERFUL TOOLKIT (SPTK)
            CDialog.cpp  -  description
                                                                                       -------------------
            begin                : Fri Mar 31 2000
            copyright            : (C) 2000-2005 by Alexey S.Parshin
            email                : alexeyp@gmail.com
***************************************************************************/

/***************************************************************************
         This library is free software; you can redistribute it and/or modify it 
         under the terms of the GNU Library General Public License as published by
         the Free Software Foundation; either version 2 of the License, or (at 
         your option) any later version.

         This library is distributed in the hope that it will be useful, but 
         WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
         General Public License for more details.

         You should have received a copy of the GNU Library General Public License
         along with this library; if not, write to the Free Software Foundation, 
         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

         Please report all bugs and problems to "alexeyp@gmail.com"
   ***************************************************************************/

#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include <string.h>
#include <stdio.h>
#include <sptk/CDatabase.h>
#include <sptk/CMessageDialog.h>
#include <sptk/CException.h>
#include <sptk/CInput.h>
#include <sptk/CQuery.h>
#include <sptk/CRegistry.h>
#include <sptk/CTabs.h>
#include <sptk/CCheckButtons.h>
#include <sptk/CRadioButtons.h>

static const unsigned headerHeight = 20;

//==============================================================================
Fl_Group * CDialogTabs::adjustPage(Fl_Group *page) {
	if (children() == 1) {
		box(FL_THIN_DOWN_FRAME);
		showTabs(false);
		page->box(FL_NO_BOX);
	} else {
		box(FL_THIN_UP_BOX);
		showTabs(true);
		page->box(FL_FLAT_BOX);
	}
	return page;
}

Fl_Group * CDialogTabs::newPage(const char *label,bool autoColor) {
	return adjustPage(CTabs::newPage(label,autoColor));
}

Fl_Group * CDialogTabs::newPage(const char *label,Fl_Color color) {
	Fl_Group * grp = adjustPage(CTabs::newPage(label,false));
	grp->color(color);
	return grp;
}

Fl_Group * CDialogTabs::newScroll(const char *label,bool autoColor) {
	if (!children()) {
		box(FL_THIN_DOWN_BOX);
		showTabs(false);
	} else {
		box(FL_THIN_UP_BOX);
		showTabs(true);
	}
	Fl_Group *grp = CTabs::newScroll(label,autoColor);
	return grp;
}

Fl_Group * CDialogTabs::newScroll(const char *label,Fl_Color color) {
	if (!children()) {
		box(FL_THIN_DOWN_BOX);
		showTabs(false);
	} else {
		box(FL_THIN_UP_BOX);
		showTabs(true);
	}
	Fl_Group *grp = CTabs::newScroll(label,false);
	grp->color(color);
	return grp;
}
//==============================================================================
CDialog::CDialog(int w, int h, const char *label)
: CWindow(w,h,label) {
	m_queriesBuilt = false;
	m_controlsScanned = false;
	m_keyValue = -1;
	m_defaultButton = NULL;

	fl_cursor(FL_CURSOR_WAIT);
	Fl::check();

	m_selectQuery = new CQuery;
	m_updateQuery = new CQuery;
	m_insertQuery = new CQuery;

	m_buttonGroup = new CGroup("",20,SP_ALIGN_BOTTOM);
	m_okButton = new CButton(SP_OK_BUTTON);
	m_cancelButton = new CButton(SP_CANCEL_BUTTON);
	m_buttonGroup->end();

	m_pages = new CDialogTabs;

	defaultButton(m_okButton);

	newPage("");

	m_modalResult = DMR_NONE;
}

CDialog::~CDialog() {
	delete m_selectQuery;
	delete m_updateQuery;
	delete m_insertQuery;
}

int CDialog::handle(int event) {
	int rc = Fl_Group::handle(event);
	if (rc) return rc;
	switch (event) {
		case FL_KEYBOARD:
			if (Fl::event_key() == FL_Enter) {
				Fl_Button *btn = dynamic_cast<Fl_Button *>(Fl::focus());
				if (btn)
					btn->do_callback();
				else if (m_defaultButton)
					m_defaultButton->do_callback();
				return 1;
			}
			break;
	}
	return 0;
}

void CDialog::defaultButton(CButton *newDefaultButton) {
	unsigned cnt = m_buttonGroup->children();
	for (unsigned bi = 0; bi < cnt; bi++) {
		CButton *button = (CButton *)m_buttonGroup->child(bi);
		button->defaultButton(button == newDefaultButton);
	}
	m_defaultButton = newDefaultButton;
}

bool CDialog::okPressed() {
	try {
		save();
	}
	catch (CException& e) {
		alert(e.text());
		return false;
	}
	return true;
}

bool CDialog::cancelPressed() {
	return true;
}

bool CDialog::showModal() {
	resize(x(),y(),w(),h());
	Fl_Widget *pressed;
	int x = (Fl::w() - w()) / 2;
	int y = (Fl::h() - h()) / 2;
	position(x,y);
	set_modal();
	fl_cursor(FL_CURSOR_DEFAULT);
	show();
	bool rc = false;

	m_modalResult = DMR_NONE;

	while (m_modalResult == DMR_NONE) {
		pressed = Fl::readqueue();

		if (!pressed) Fl::wait(0.1);
		else {
			fl_cursor(FL_CURSOR_WAIT);
			Fl::check();
			if (pressed == this || pressed == m_cancelButton) {
				if (cancelPressed()) {
					m_modalResult = DMR_CANCEL;
					break;
				}
			}
			if (pressed == m_okButton) {
				if (okPressed()) {
					rc = true;
					m_modalResult = DMR_OK;
					break;
				}
			}
			fl_cursor(FL_CURSOR_DEFAULT);
		}
	}

	if (m_modalResult == DMR_USER)
		rc = true;

	fl_cursor(FL_CURSOR_DEFAULT);
	hide();
	Fl::check();
	return rc;
}

void CDialog::database(CDatabase *db) {
	m_selectQuery->database(db);
	m_updateQuery->database(db);
	m_insertQuery->database(db);
}

CDatabase * CDialog::database() const {
	return m_selectQuery->database();
}

void CDialog::table(const CString tableName) {
	if (m_tableName != tableName) {
		m_tableName = tableName;
		m_queriesBuilt = false;
	}
}

void CDialog::table(CDatabase *db,const CString tb,const CString key) {
	database(db);
	table(tb);
	keyField(key);
}

void CDialog::keyField(const CString fldName) {
	if (m_keyField != fldName) {
		m_keyValue = -1;
		m_keyField = fldName;
		m_queriesBuilt = false;
	}
}

void CDialog::keyValue(int val) {
	m_keyValue = val;
	try {
		load();
	}
	catch (CException& e) {
		m_keyValue = -1;
		alert(e.text());
	}
}

bool CDialog::buildQueries() {
	CString columnNames;
	CString paramNames;
	CString updateNames;

	if (!m_controlsScanned) scanControls();

	if (m_queriesBuilt) return true;

	unsigned fcnt = m_defaultFields.count();
	for (unsigned bi = 0; bi < fcnt; bi++) {
		const CControl& control = m_defaultFields[bi];
		const CString& fieldName = control.fieldName();
		if (bi) {
			columnNames += "," + fieldName;
			paramNames  += ",:" + fieldName;
			updateNames += "," + fieldName + "=:" + fieldName;
		} else {
			columnNames = fieldName;
			paramNames  = ":" + fieldName;
			updateNames = fieldName + "=:" + fieldName;
		}
	}

	m_selectQuery->sql("SELECT "+columnNames+" FROM "+m_tableName+" WHERE "+m_keyField+"=:key");
	m_insertQuery->sql("INSERT INTO "+m_tableName+"("+columnNames+") VALUES ("+paramNames+")");
	m_updateQuery->sql("UPDATE "+m_tableName+" SET "+updateNames+" WHERE "+m_keyField+"=:key");

	m_queriesBuilt = true;

	return true;
}

bool CDialog::load() {
	buildQueries();
	reset();
	if (m_keyValue > 0) {
		m_selectQuery->param("key").setInteger(m_keyValue);
		m_selectQuery->open();
		unsigned cnt = m_defaultFields.count();
		for (unsigned bi = 0; bi < cnt; bi++) {
			try {
				CControl& control = m_defaultFields[bi];
				control.load(m_selectQuery);
			}
			catch (CException& e) {
				alert(e.text());
			}
		}
		m_selectQuery->close();
	}
	return true;
}

bool CDialog::save() {

	if (!m_controlsScanned) scanControls();

	unsigned cnt = m_defaultFields.count();
	if (!cnt) return true;

   // Validating data in controls
	for (unsigned bi = 0; bi < cnt; bi++) {
		CControl& control = m_defaultFields[bi];
		if (!control.valid()) {
			Fl::focus(control.control());
			throw CException(control.label() + " entry has the incorrect value!");
		}
		if (control.flags() & FGE_MANDATORY) {
			CString test = control.data();
			if (!test.trim().length()) {
				Fl::focus(control.control());
				throw CException(control.label() + " entry can't be empty!");
			}
		}
	}

	if (!database()) return true;
	buildQueries();
	CQuery *query = m_insertQuery;

	if (m_keyValue > 0) {
		query = m_updateQuery;
		query->param("key").setInteger(m_keyValue);
	}

         //puts(query->sql());

	for (unsigned ci = 0; ci < cnt; ci++) {
		CControl& control = m_defaultFields[ci];
		control.save(query);
	}
	query->exec();

	return true;
}

bool CDialog::loadFromRegistry(CRegistry *registry,const char *section) {
	registry->topic(section);
	unsigned cnt = m_allFields.count();
	for (unsigned bi = 0; bi < cnt; bi++) {
		CControl& control = m_allFields[bi];
		CString   stringData;
		CString   fieldName = control.fieldName();
		try {
			stringData = (*registry)[fieldName.c_str()];
			control.data(stringData);
		}
		catch (...) {
			continue;
		}
	}
	return true;
}

bool CDialog::saveToRegistry(CRegistry *registry,const char *section) {
	registry->topic(section);
	registry->topic().clear();
	unsigned cnt = m_allFields.count();
	for (unsigned bi = 0; bi < cnt; bi++) {
		CControl& control = m_allFields[bi];
		CString   stringData;
		CString   fieldName = control.fieldName();
		stringData = control.data();
		stringData = stringData.trim();
		if (stringData.length())
			(*registry)[fieldName.c_str()] = stringData;

	}
	return true;
}

CControl& CDialog::operator [] (unsigned index) {
	if (!m_controlsScanned) scanControls();
	if (index >= m_allFields.count())
		throw CException("Field index <"+CString(index)+" is invalid.");
	return m_allFields[index];
}

CControl& CDialog::operator [] (CString fieldName) {
	if (!m_controlsScanned) scanControls();
	return m_allFields[fieldName];
}

CButton *CDialog::addExtraButton(CButtonKind buttonKind,const char *label,Fl_Callback_p callbackFunction) {
	m_buttonGroup->begin();
	CButton *extraButton = new CButton(buttonKind,SP_ALIGN_RIGHT,label);
	m_buttonGroup->end();
	if (callbackFunction)
		extraButton->callback(callbackFunction);
	return extraButton;
}

int CDialog::keyValue() const {
	return m_keyValue;
}

void CDialog::alert(const CString s) const {
	spWarning(s.c_str());
}

void CDialog::scanControls() {
	m_allFields.clear();
	m_defaultFields.clear();
	m_allFields << *m_pages;
	m_defaultFields = m_allFields;
	m_defaultFields.remove(m_specialProcessingControls);
	m_controlsScanned = true;
	m_queriesBuilt = false;
}

Fl_Group* CDialog::newPage(const char *label,bool autoColor) {
	return m_pages->newPage(label,autoColor);
}

Fl_Group* CDialog::newPage(const char *label,Fl_Color color) {
	return m_pages->newPage(label,color);
}

Fl_Group* CDialog::newScroll(const char *label,bool autoColor) {
	return m_pages->newScroll(label,autoColor);
}

Fl_Group* CDialog::newScroll(const char *label,Fl_Color color) {
	return m_pages->newScroll(label,color);
}

bool CDialog::reset() {
	if (!m_controlsScanned) scanControls();
	m_allFields.reset();
	return true;
}
