/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFieldList.cpp  -  description
                             -------------------
    begin                : 20 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <string.h>

#include <sptk/CException.h>
#include <sptk/CFieldList.h>

CFieldList::CFieldList(bool indexed)  { 
	m_userData = 0;
	if (indexed) {
		m_index = new CStringMap<CField *>;
	} else {
		m_index = 0L;
	}
}

CFieldList::~CFieldList() { 
	clear(); 
	if (m_index) 
		delete m_index; 
}

void CFieldList::clear() {
	unsigned cnt = m_list.count();
	for (unsigned i = 0; i < cnt; i++) {
		CField *field = (CField *)m_list[i];
		delete field;
	}
	m_list.clear();
	if (m_index)
		m_index->clear();
}

CField& CFieldList::add(const char *fname) {
	bool duplicate = false;
	if (m_index) {
		CField **pfld = m_index->find(fname);
		if (pfld)
			duplicate = true;
	} else {
      try {
		   CField *pfld = fieldByName(fname);
		   duplicate = (pfld != 0L);
      }
      catch (...) {
      }
	}

	if (duplicate)
		throw CException("Attempt to duplicate field name");

	CField *field = new CField(fname);

	m_list.add(field);

	if (m_index)
		m_index->insert(fname,field);

	return *field;
}

CField& CFieldList::add(CField *field) {
	m_list.add(field);

	if (m_index)
		m_index->insert(field->m_name.c_str(),field);

	return *field;
}

CField *CFieldList::fieldByName(const char *fname) const {
	if (m_index) {
		CField **pfld = m_index->find(fname);
		if (pfld)
			return *pfld;
	} else {
		unsigned cnt = m_list.count();
		for (unsigned i = 0; i < cnt; i++) {
			CField *field = (CField *)m_list[i];
			if (strcmp(field->m_name,fname) == 0)
				return field;
		}
	}
	throw CException("Field name '" + CString(fname) + "' not found");
}
