/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBaseIntMap.cpp  -  description
                             -------------------
    begin                : 13 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CIntMap.h>
#include <sptk/CException.h>

CBaseIntMap::~CBaseIntMap() {
	clear();
}

void CBaseIntMap::clear() { 
	unsigned cnt = m_items.count();
	for (unsigned i = 0; i < cnt; i++) {
		CBaseIntMapItem *currentItem = (CBaseIntMapItem *) m_items[i];
		deleteData(currentItem->m_data);
		delete (CBaseIntMapItem *) currentItem;
	}
	m_items.clear(); 
}

int CBaseIntMap::integersSearch(const void *key, const void *item) {
	CBaseIntMapItem *c = *(CBaseIntMapItem **)item;
	return *(int *)key - c->m_id;
}

int passCount;
int searchCount;

void *CBaseIntMap::insertItem(int id,void *data) {
	searchCount++;

	CBaseIntMapItem *currentItem = 0L;
	int compareResult = 0;
	int cnt = m_items.count();
	int pos = cnt / 2;
	int step = pos;
	if (!step) step = 1;
	if (cnt)
		do {
			currentItem = (CBaseIntMapItem *)m_items[pos];
			compareResult = id - currentItem->m_id;
			passCount++;
			if (compareResult == 0) {
            // found existing item
				deleteData(currentItem->m_data);
				currentItem->m_data = data;
            return currentItem;
			}
			if (compareResult < 0) {
				if (step > 0)
					step /= -2;
				if (step == 0) {
					pos--;
					break;
				}
			}
			else if (compareResult > 0) {
				if (step < 0)
					step /= -2;
				if (step == 0) {
					pos++;
					break;
				}
			}
			pos += step;
			if (pos < 0) {
				step /= -2;
				pos = 0;
			}
			else
				if (pos >= cnt) {
					step /= -2;
					pos = cnt - 1;
				}
		} while (step);

	CBaseIntMapItem *newItem = new CBaseIntMapItem(id,data);

	if (pos < 0) {
		m_items.insert(0,newItem);
      return newItem;
	}

	if (pos >= cnt) {
		m_items.insert(cnt,newItem);
      return newItem;
	}

	currentItem = (CBaseIntMapItem *)m_items[pos];
	compareResult = id - currentItem->m_id;

	passCount++;

	if (compareResult < 0) {
		m_items.insert(pos,newItem);
	} else {
		m_items.insert(pos+1,newItem);
	}
   return newItem;
}

void *CBaseIntMap::insert(int id,void *data) {
	return insertItem(id,data);
}

void CBaseIntMap::remove(int id) {
	CBaseIntMapItem *item = findItem(id);
	if (!item) return;
	deleteData(item->m_data);
	m_items.remove(item);
}

CBaseIntMapItem *CBaseIntMap::findItem(int id) const {
	unsigned    cnt = m_items.count();
	const void *base = m_items.base();
   void       *ptr = 0;
   if (cnt)
	   ptr = bsearch((void *)&id,base,cnt,sizeof(void *),integersSearch);
	if (!ptr) return 0L;
	return *(CBaseIntMapItem **)ptr;
}

void *CBaseIntMap::find(int id) const {
	CBaseIntMapItem *item = findItem(id);
	if (!item) return 0;
	return item->m_data;
}

void CBaseIntMap::throwNotFoundException(int id) const {
	throw CException("Item for id="+CString(id)+" not found.");
}
//=============================================================================
CBaseIntMapIterator::CBaseIntMapIterator(const CBaseIntMap& map) {
	m_map = &map;
	m_position = 0;
}

void *CBaseIntMapIterator::first() {
	m_position = 0;
	return operator * ();
}

void *CBaseIntMapIterator::last() {
	m_position = m_map->count() - 1;
	return operator * ();
}

void *CBaseIntMapIterator::operator *  () {
	if (m_position >= m_map->count()) 
		return 0L;
	CBaseIntMapItem *item = (CBaseIntMapItem *)m_map->m_items[m_position];
	return item->m_data;
}

int CBaseIntMapIterator::ident() const {
	if (m_position >= m_map->count()) 
		return 0L;
	CBaseIntMapItem *item = (CBaseIntMapItem *)m_map->m_items[m_position];
	return item->m_id;
}

void *CBaseIntMapIterator::operator ++ (int) {
	if (m_position < m_map->count())
		m_position++;
	return operator * ();
}

void *CBaseIntMapIterator::operator -- (int) {
	m_position--;
	return operator * ();
}
