/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          clist.cpp  -  description
                             -------------------
    begin                : Fri May 26 2000
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CList.h>
#include <stdlib.h>
#include <string.h>

CList::CList(unsigned sz) {
   m_items = NULL;
   m_capacity = 0;
   resize(sz);
}

CList::~CList() {
   if (m_items) free(m_items);
}

void CList::resize(unsigned newsize) {
   unsigned cap = (newsize * 9 / 64 + 1) * 8;
   if (cap != m_capacity) {
      m_capacity = cap;
      if (m_items)
         m_items = (PListItem *)realloc(m_items,m_capacity * sizeof(PListItem));
      else m_items = (PListItem *)malloc(m_capacity * sizeof(PListItem));
   }
   m_size = newsize;
}

void CList::add(PListItem item) {
   if ( m_size == m_capacity ) {
      resize(m_size + 1);
      m_items[m_size - 1] = item;
   } else {
      m_items[m_size++] = item;
   }
}

void CList::addList(const CList& itemList) {
   if (!itemList.m_size)
      return;
   unsigned oldSize = m_size; 
   unsigned newSize = m_size + itemList.m_size;
   if ( newSize >= m_capacity ) 
      resize(newSize + 1);
   memcpy(m_items+oldSize,itemList.m_items,itemList.m_size * sizeof(PListItem));
}

void CList::insert(unsigned pos,PListItem item) {
   if ( m_size == m_capacity )
      resize(m_size + 1);
   else  m_size++;
   int mvSize = (m_size-1-pos);
   if (mvSize > 0)
      memmove(m_items+pos+1,m_items+pos,mvSize*sizeof(PListItem));
   m_items[pos] = item;
}

void CList::remove(unsigned pos) {
   int mvSize = (m_size-1-pos);
   if (mvSize > 0)
      memmove(m_items+pos,m_items+pos+1,mvSize*sizeof(PListItem));
   resize(m_size - 1);
}

bool CList::remove(void *p) {
   for (unsigned pos = 0; pos < count(); pos++)
      if (m_items[pos] == p) {
         remove(pos);
         return true;
      }
   return false;
}

bool CList::removeList(const CList& itemList) {
   unsigned cnt = itemList.count();
   for (unsigned pos = 0; pos < cnt; pos++) {
      remove(itemList[pos]);
   }
   return true;
}

void CList::sort(int (*sort_function)(const void *, const void *)) {
   if (m_size)
      qsort(m_items,m_size,sizeof(PListItem),sort_function);
}

int CList::indexOf(const void * p) const {
   unsigned cnt = count();
   for (unsigned pos = 0; pos < cnt; pos++)
      if (m_items[pos] == p)
         return pos;
   return -1;
}

int CList::foreach(CForeachFunction todo,void * arg) {
   unsigned cnt = count();
   for (unsigned pos = 0; pos < cnt; pos++)
      if (todo(m_items[pos],arg))
         return pos;
   return -1;
}

int CList::foreach(CForeachFunction todo,void * arg) const {
   unsigned cnt = count();
   for (unsigned pos = 0; pos < cnt; pos++)
      if (todo(m_items[pos],arg))
         return pos;
   return -1;
}
