/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          clistviewrows.cpp  -  description
                             -------------------
    begin                : Tue Jul 23 2002
    copyright            : (C) 2002-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include <sptk/CListViewRows.h>
#include <sptk/CException.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

CListViewRows::CListViewRows() {
   m_sortColumn = -1;
   m_sortAscending = true;
   m_sortColumnType = VAR_STRING;
   m_fullHeight = 0;
}

CListViewRows::~CListViewRows() {
   clear();
}

void CListViewRows::truncate(unsigned cnt) {
   unsigned rowCount = m_rows.count();
   if (cnt < rowCount) {
      for (unsigned i = cnt; i < rowCount; i++) {
         CPackedStrings *row = (CPackedStrings *)m_rows[i];
         m_fullHeight -= row->height;
         delete (CPackedStrings *)row;
      }
      m_rows.resize(cnt);
   }
}

unsigned CListViewRows::add(CPackedStrings *ss) {
   int lineNumber = count();
   m_rows.add(ss);
   m_fullHeight += ss->height;
   return lineNumber;
}

unsigned CListViewRows::insert(unsigned position,CPackedStrings *ss) {
   m_rows.insert(position,ss);
   m_fullHeight += ss->height;
   return position;
}

unsigned CListViewRows::update(unsigned index,CPackedStrings *ss) {
   CPackedStrings *s = (CPackedStrings *)m_rows[index];
   int oldh = s->height;
   if (s) delete s;
   m_fullHeight += ss->height - oldh;
   m_rows[index] = ss;
   return index;
}

void CListViewRows::clear() {
   for (unsigned i = 0; i < m_rows.count(); i++)
      delete (CPackedStrings *) m_rows[i];
   m_rows.clear();
   m_fullHeight = 0;
}

void CListViewRows::remove(unsigned index) {
   if (index < m_rows.count()) {
      CPackedStrings *row = (CPackedStrings *)m_rows[index];
      m_fullHeight -= row->height;
      delete row;
      m_rows.remove(index);
   }
}

int CListViewRows::currentSortColumn;

int CListViewRows::compare_strings(const void *a, const void *b) {
   CPackedStrings *aa = *(CPackedStrings **)a;
   CPackedStrings *bb = *(CPackedStrings **)b;
   return strcmp((*aa)[currentSortColumn],(*bb)[currentSortColumn]);
}

int CListViewRows::compare_integers(const void *a, const void *b) {
   CPackedStrings *aa = *(CPackedStrings **)a;
   CPackedStrings *bb = *(CPackedStrings **)b;
   int i1 = atoi((*aa)[currentSortColumn]);
   int i2 = atoi((*bb)[currentSortColumn]);
   return i1 - i2;
}

int CListViewRows::compare_floats(const void *a, const void *b) {
   CPackedStrings *aa = *(CPackedStrings **)a;
   CPackedStrings *bb = *(CPackedStrings **)b;
   double d1 = atof((*aa)[currentSortColumn]);
   double d2 = atof((*bb)[currentSortColumn]);
   if (d1 > d2) return 1;
   if (d1 < d2) return -1;
   return 0;
}

int CListViewRows::compare_dates(const void *a, const void *b) {
   CPackedStrings *aa = *(CPackedStrings **)a;
   CPackedStrings *bb = *(CPackedStrings **)b;
   CDateTime d1((*aa)[currentSortColumn]);
   CDateTime d2((*bb)[currentSortColumn]);
   if (d1 > d2) return 1;
   if (d1 < d2) return -1;
   return 0;
}

int CListViewRows::compare_datetimes(const void *a, const void *b) {
   CPackedStrings *aa = *(CPackedStrings **)a;
   CPackedStrings *bb = *(CPackedStrings **)b;
   CDateTime d1((*aa)[currentSortColumn]);
   CDateTime d2((*bb)[currentSortColumn]);
   if (d1 > d2) return 1;
   if (d1 < d2) return -1;
   return 0;
}

void CListViewRows::sort() {
   fl_cursor(FL_CURSOR_WAIT);
   Fl::check();

   int m_size = m_rows.count();
   if (m_sortColumn >= 0 && m_size > 1) {
      CListViewRows::currentSortColumn = m_sortColumn;
      switch (m_sortColumnType) {
         case VAR_INT:
            m_rows.sort(compare_integers);
            break;
         case VAR_FLOAT:
            m_rows.sort(compare_floats);
            break;
         case VAR_DATE: 
            m_rows.sort(compare_dates);
            break;
         case VAR_DATE_TIME: 
            m_rows.sort(compare_datetimes);
            break;
         default:
            m_rows.sort(compare_strings);
            break;
      }
      if (!m_sortAscending) {
         // reversing sort order for the descending sort
         unsigned cnt = m_rows.count();
         unsigned mid = cnt / 2;
         unsigned j = cnt - 1;
         for (unsigned i = 0; i < mid; i++, j--) {
            PListItem item = m_rows[i];
            m_rows[i] = m_rows[j];
            m_rows[j] = item;
         }
      }
   }
   fl_cursor(FL_CURSOR_DEFAULT);
   Fl::check();
}

void CListViewRows::sortColumn(int column,CVariantType columnType,bool sortNow) {
   m_sortColumn = column;
   m_sortColumnType = columnType;
   if (sortNow) sort();
}

void CListViewRows::sortAscending(bool ascending,bool sortNow) {
   m_sortAscending = ascending;
   if (sortNow) sort();
}

int CListViewRows::indexOf(CPackedStrings * ss) const {
   return m_rows.indexOf(ss);
}

void CListViewRows::saveToFile(const char *fname) {
   unsigned rowsCount = m_rows.count();
   if (!rowsCount) return;
   unsigned colsCount = ((CPackedStrings *)m_rows[0])->count();
   FILE *f = fopen(fname,"w+t");
   if (!f)
      throw CException("Can't open the file");
   for (unsigned i = 0; i < rowsCount; i++) {
      CPackedStrings& ps = *(CPackedStrings *)m_rows[i];
      for (unsigned j = 0; j < colsCount; j++) {
         fprintf(f,"%s|",ps[j]);
      }
      fprintf(f,"\n");
   }
   fclose(f);
}
