/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMailMessageBody.cpp  -  description
                             -------------------
    begin                : 25 Oct 2004
    copyright            : (C) 2000-2005 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CMailMessageBody.h>
#include <sptk/CStringMap.h>
#include <sptk/CStringList.h>

/// A map of the HTML tags to remove or process
class CHtmlTagsToProcess : public CStringMap<int> {
public:
	CHtmlTagsToProcess();
};

static CHtmlTagsToProcess tagsToProcess;

#define REMOVE_THIS_TAG 1
#define PROCESS_THIS_TAG 2

CHtmlTagsToProcess::CHtmlTagsToProcess() : CStringMap<int>(true) {
	insert("HTML",REMOVE_THIS_TAG);
	insert("BODY",REMOVE_THIS_TAG);
	insert("BASE",REMOVE_THIS_TAG);
	insert("CENTER",REMOVE_THIS_TAG);
	insert("HEAD",REMOVE_THIS_TAG);
	insert("TABLE",REMOVE_THIS_TAG);
	insert("TBODY",REMOVE_THIS_TAG);
	insert("TH",REMOVE_THIS_TAG);
	insert("TR",REMOVE_THIS_TAG);
	insert("TD",REMOVE_THIS_TAG);
	insert("DIV",REMOVE_THIS_TAG);
	insert("SPACER",REMOVE_THIS_TAG);
	insert("SPAN",REMOVE_THIS_TAG);
	insert("P",REMOVE_THIS_TAG);
	insert("BR",REMOVE_THIS_TAG);
	insert("SCRIPT",REMOVE_THIS_TAG);
	insert("NOSCRIPT",REMOVE_THIS_TAG);
	insert("NOBR",REMOVE_THIS_TAG);
	insert("FONT",REMOVE_THIS_TAG);
	insert("FORM",REMOVE_THIS_TAG);
	insert("INPUT",REMOVE_THIS_TAG);
	insert("H4",REMOVE_THIS_TAG);
	insert("H3",REMOVE_THIS_TAG);
	insert("H2",REMOVE_THIS_TAG);
	insert("H1",REMOVE_THIS_TAG);
	insert("STRONG",REMOVE_THIS_TAG);
	insert("B",REMOVE_THIS_TAG);
	insert("A",PROCESS_THIS_TAG);
	insert("IMG",PROCESS_THIS_TAG);
	insert("FRAME",REMOVE_THIS_TAG);
	insert("IFRAME",REMOVE_THIS_TAG);
	insert("FRAMESET",REMOVE_THIS_TAG);
}

CString CMailMessageBody::processTag(CString tag,CString params) {
	if (tag.lowerCase() == "a") {
		int pos = params.lowerCase().pos("src=");
		if (pos < 0) return "";
		char *p = (char*) params.c_str() + pos + 4;
		char *e = strpbrk(p," >/");
		if (e) *e = 0;
		return e;
	}
	return "";
}

CString CMailMessageBody::stripHtml(const CString& origHtml) {
	CStringList html(origHtml,"<");
	unsigned    i = 0;

   // Remove comments and scripts
	const char *tagEnd = 0L;
	for (i = 0; i < html.count(); i++) {
		CString& tag = html[i];
		if (tagEnd) {
			if (tag.lowerCase().pos(tagEnd) >= 0)
				tagEnd = 0;
			html.remove(i);
			i--;
			continue;
		}
		CString tagLowerCase(tag.lowerCase());
		if (tagLowerCase.pos("script") == 0) {
			tagEnd = "/script>";
			html.remove(i);
			i--;
			continue;
		}
		if (tagLowerCase.pos("style") == 0) {
			tagEnd = "/style>";
			html.remove(i);
			i--;
		}
		if (tagLowerCase.pos("!--") == 0) {
			tagEnd = "-->";
			html.remove(i);
			i--;
		}
		if (tagLowerCase.pos("head") == 0) {
			tagEnd = "head>";
			html.remove(i);
			i--;
		}
	}

	for (i = 0; i < html.count(); i++) {
		char *start = (char *) html[i].c_str();
		char *end = strpbrk(start," >");
		char *params = NULL;
		if (!end) {
			html.remove(i);
			i--;
			continue;
		}
		if (*end == ' ') 
			params = end + 1;
		*end = 0;
		if (*start == '/') start++;
		int *mode = tagsToProcess.find(start);
		if (params) 
			end = strchr(end + 1,'>');
		if (!end) {
			html.remove(i);
			i--;
			continue;
		}
		if (mode) {
			CString text;
			if (*mode == REMOVE_THIS_TAG || !params)
				text = end + 1;
			else {
				text = processTag(start,params) + CString(end + 1);
			}
			html[i] = text.trim().replace("&nbsp;"," ").replace("     "," ").replace("   "," ");
		}
		if (i && html[i].trim().length() == 0) {
			html.remove(i);
			i--;
		}
	}

	return html.asString(" ");
}

void CMailMessageBody::text(const CString& messageText) {
	if (messageText.subString(0,100).upperCase().pos("<HTML>") < 0) {
		m_type = MMT_PLAIN_TEXT_MESSAGE;
		m_plainText = messageText;
		m_htmlText = "";
	} else {
		m_type = MMT_HTML_MESSAGE;
		m_plainText = stripHtml(messageText);
		m_htmlText = messageText;
	}
}

