/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CParams.cpp  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CParams.h>
#include <sptk/CIntList.h>
#include <sptk/CException.h>

#include <string.h>
#include <ctype.h>

static char *lowerCase(char * src) {
   char * ptr = src;
   while ( *ptr ) {
      *ptr = (char) tolower( *ptr );
      ptr++;
   }
   return src;
}
//---------------------------------------------------------------------------
CParam::CParam(char *name) : CVariant() {
   lowerCase(name);
   m_name = name;
   m_timeData = new char[64];
   m_callbackLength = 0;
}
//---------------------------------------------------------------------------
CParam::~CParam() {
   delete [] m_timeData;
}
//---------------------------------------------------------------------------
CParam& CParam::operator = (const CParam& param) {
   this->setData(param);
   this->m_bindParamIndexes = param.m_bindParamIndexes;
   return *this;
}
//---------------------------------------------------------------------------
void CParam::bindAdd(unsigned bindIndex) {
   m_bindParamIndexes.add(bindIndex);
}
//---------------------------------------------------------------------------
CParamList::~CParamList() {
   for (unsigned i = 0; i < count(); i++) {
      CParam *item = (CParam *)m_items[i];
      delete item;
   }
}
//---------------------------------------------------------------------------
void CParamList::add(const CParam * item) {
   m_items.add((void *)item);
}
//---------------------------------------------------------------------------
CParam& CParamList::operator [] (unsigned index) const {
   if (index >= m_items.count())
      throw CException("Invalid parameter index",__FILE__,__LINE__);
   return *(CParam *)m_items[index];
}
//---------------------------------------------------------------------------
CParam& CParamList::operator [] (int index) const {
   if ((unsigned)index >= m_items.count())
      throw CException("Invalid parameter index",__FILE__,__LINE__);
   return *(CParam *)m_items[index];
}
//---------------------------------------------------------------------------
int CParamList::paramIndex(const char *paramName) {
   for (unsigned i = 0; i < count(); i++) {
      CParam *item = (CParam *)m_items[i];
      if ( item->name() == paramName )
         return i;
   }
   return -1;
}
//---------------------------------------------------------------------------
CParam& CParamList::operator [] (const char *paramName) const {
   CString pname = CString(paramName).lowerCase();
   for (unsigned i = 0; i < count(); i++) {
      CParam *item = (CParam *)m_items[i];
      if ( item->name() == pname )
         return *item;
   }
   throw CException("Invalid parameter name: "+CString(paramName),__FILE__,__LINE__);
}
//---------------------------------------------------------------------------
CParam& CParamList::operator [] (const CString& paramName) const {
   return operator[] (paramName.c_str());
}
//---------------------------------------------------------------------------
void CParamList::remove(unsigned index) {
   CParam *item = (CParam *)m_items[index];
   delete item;
   m_items.remove(index);
}
//---------------------------------------------------------------------------
