/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRegistry.cpp  -  description
                             -------------------
    begin                : Tue Mar 26 2002
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sptk/CRegistry.h>
#include <sptk/CException.h>
//---------------------------------------------------------------------------
CRegistryItem::CRegistryItem(const CString buffer) {
   if (!buffer.length()) return;
   char *p = strchr((char *)buffer.c_str(),'=');
   if (p) {
      *p = 0;
      p++;
      value = p;
   }
   m_name = buffer.trim();
}
//----------------------------------------------------------------------------
void CRegistryTopic::addItem(const CString buffer) {
   CRegistryItem *item = new CRegistryItem(buffer);
   insert(item->name(),item);
}
// ----------------------------------------------------------------------------
CString CRegistry::homeDirectory() {
#ifndef _WIN32
   CString homeDir = getenv("HOME");
   if (homeDir.trim().length())
      homeDir += "/";
#else
   char *hdrive = getenv("HOMEDRIVE");
   char *hdir   = getenv("HOMEPATH");
   if (!hdir && !hdrive) 
      return getenv("WINDIR") + CString("\\");

   CString homeDrive;
   CString homeDir;
   if (hdrive)
      homeDrive = hdrive;
   if (hdir)
      homeDir = hdir;
   if (homeDir == "\\")
      homeDir = homeDrive + "\\";
   else homeDir = homeDrive + "\\" + homeDir;
#endif
   return homeDir;
}

CRegistry::CRegistry(const char *fileName,CRegistryMode mode)
: m_fileName(fileName) {
   if (m_fileName.length() && mode == USER_REGISTRY) {
      m_fileName = homeDirectory() + m_fileName;
   }
   topic("Default");
}

CRegistry::~CRegistry() {
   clear();   
}

void CRegistry::load(CStringList& inputData) {
   for (unsigned row = 0; row < inputData.count(); row++) {
      char *b = (char *)inputData[row].c_str();
      int i = strlen(b) - 1;
      while (i >= 0 && (unsigned char)b[i] < 33) i--;
      b[i+1] = 0;

      if (i <= 0) continue;

      if (b[0] == '[' && b[i] == ']') {
         b[i] = 0;
         topic(b+1);
         continue;
      }

      m_currentTopic->addItem((char *)b);
   }
}

void CRegistry::load() {
   CStringList inputData;
   if (!inputData.loadFromFile(m_fileName))
      throw CException("Can't read file '" + m_fileName + "'.",__FILE__,__LINE__);
   load(inputData);
}

void CRegistry::save(CStringList& outputData) {
   outputData.clear();
   for (unsigned i=0; i < m_topics.count(); i++) {
      CRegistryTopic *topic = (CRegistryTopic *)m_topics[i];
      if (!topic->count()) continue;
      CString topicName = "[" + topic->name() + "]";
      outputData.add(topicName.c_str());
      CRegistryTopic::Iterator itor(*topic);
      for (; *itor; itor++) {
         CRegistryItem *item = *itor;
         CString itemData = item->name() + "=" + item->value;
         outputData.add(itemData.c_str());
      }
   }
}

void CRegistry::save() {
   CStringList outputData;
   save(outputData);
   if (!outputData.saveToFile(m_fileName))
      throw CException("Can't write file '" + m_fileName + "'.",__FILE__,__LINE__);
}

void CRegistry::topic(const CString atopic) {
   if (m_topic != atopic) {
      m_topic = atopic;
      unsigned topicCount = m_topics.count();
      for (unsigned i = 0; i < topicCount; i++) {
         CRegistryTopic *tp = (CRegistryTopic *) m_topics[i];
         if (tp->m_name == m_topic) {
            m_currentTopic = tp;
            return;
         }
      }
      m_currentTopic = new CRegistryTopic(m_topic);
      m_topics.add(m_currentTopic);
   }
}

const CString& CRegistry::operator[] (const char *name) const {
   CRegistryItem *item = (CRegistryItem *) m_currentTopic->find(name);
   if (!item) return m_emptyString;
   return item->value;
}

CString& CRegistry::operator[] (const char *name) {
   CRegistryItem *item = (CRegistryItem *) m_currentTopic->find(name);
   if (!item) {
      item = new CRegistryItem(name);
      m_currentTopic->insert(name,item);
   }
   return item->value;
}

void CRegistry::clear() {
   m_currentTopic = 0L;
   for (unsigned i=0; i < m_topics.count(); i++) {
      CRegistryTopic *topic = (CRegistryTopic *)m_topics[i];
      delete topic;
   }
   m_topics.clear();
}   
