/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CString.cpp  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <math.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <sptk/CString.h>
#include <sptk/CVariant.h>
#include <sptk/CFieldList.h>
#include <errno.h>
//------------------------------------------------------------------------------
// constructors & destructors
//------------------------------------------------------------------------------
CString::CString() {
	str = strdup("");
}

CString::CString(const char *s) {
	if (s)
		str = strdup(s);
	else str = strdup("");
}

CString::CString(const CString& s) {
	str = strdup(s.str);
}

CString::CString(char c,int repeater) {
	if (repeater == 1) {
		str = strdup(" ");
		str[0] = c;
	} else {
		str = (char *)malloc(repeater + 1);
		memset(str,c,repeater);
		str[repeater] = 0;
	}
}

CString::CString(int number) {
	char  buffer[16];
	sprintf(buffer,"%i",number);
	str = strdup(buffer);
}

CString::CString(unsigned number) {
	char  buffer[16];
	sprintf(buffer,"%i",number);
	str = strdup(buffer);
}

CString::CString(const char *s,int maxlen) {
	if (maxlen>=0) {
		str = (char *)malloc(maxlen + 1);
		strncpy(str,s,maxlen);
		str[maxlen] = 0;
	} else {
		str = strdup(s);
	}
}

CString::CString(const CBuffer& v) {
	str = (char *)malloc(v.bytes() + 1);
	strncpy(str,v.data(),v.bytes());
	str[v.bytes()] = 0;
}

CString::~CString() {
	free(str);
}
//------------------------------------------------------------------------------
// assignments
//------------------------------------------------------------------------------
void CString::assign(const char *s) {
	char *old = str;
	if (!s) s = "";
	str = strdup(s);
	free(old);
}

CString& CString::operator = (const char *s) {
	assign(s);
	return *this;
}

CString& CString::operator = (const CString& s) {
	assign(s.str);
	return *this;
}

CString& CString::operator = (const CBuffer& v) {
	free(str);
	str = (char *)malloc(v.bytes() + 1);
	strncpy(str,v.data(),v.bytes());
	str[v.bytes()] = 0;
	return *this;
}

CString& CString::operator = (char ch) {
	char buffer[] = { ch, 0 };
	assign(buffer);
	return *this;
}

CString& CString::operator = (int i) {
	char buffer[16];
	sprintf(buffer,"%i",i);
	assign(buffer);
	return *this;
}

CString& CString::operator = (unsigned i) {
	char buffer[16];
	sprintf(buffer,"%u",i);
	assign(buffer);
	return *this;
}

CString& CString::operator = (long i) {
	char buffer[16];
	sprintf(buffer,"%li",i);
	assign(buffer);
	return *this;
}

CString& CString::operator = (float f) {
	char buffer[32];
	sprintf(buffer,"%f",f);
	assign(buffer);
	return *this;
}

CString& CString::operator = (double d) {
	char buffer[32];
	sprintf(buffer,"%f",d);
	assign(buffer);
	return *this;
}

CString& CString::operator = (const CVariant& v) {
	assign(v.asString());
	return *this;
}

CString& CString::operator = (CField& v) {
	assign(v.asString());
	return *this;
}
//------------------------------------------------------------------------------
// comparisions
//------------------------------------------------------------------------------
bool operator <  (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) < 0;
}

bool operator <= (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) <= 0;
}

bool operator >  (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) > 0;
}

bool operator >= (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) >= 0;
}

bool operator == (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) == 0;
}

bool operator != (const CString &s1, const CString &s2) {
	return strcmp(s1.str,s2.str) != 0;
}
//------------------------------------------------------------------------------
// operations
//------------------------------------------------------------------------------
CString CString::operator + (char * s) const {
	int len1 = length();
	int len2 = strlen(s);

	CString s1(str,len1 + len2);
	strcat(s1.str,s);

	return s1;
}

CString CString::operator + (const char * s) const {
	int len1 = length();
	int len2 = strlen(s);

	CString s1(str,len1 + len2);
	strcat(s1.str,s);

	return s1;
}

CString CString::operator + (const CString& s) const {
	int len1 = length();
	int len2 = s.length();

	CString s1(str,len1 + len2);
	strcat(s1.str,s.str);

	return s1;
}

CString CString::operator + (char ch) const {
	int len = length();

	CString s1(str,len + 1);
	s1.str[len] = ch;
	s1.str[len+1] = 0;

	return s1;
}
//------------------------------------------------------------------------------
CString& CString::operator += (const char * s) {
	str = (char *) realloc(str, length() + strlen(s) + 1);
	strcat(str,s);
	return *this;
}

CString& CString::operator += (const CString& s) {
	str = (char *) realloc(str, length() + s.length() + 1);
	strcat(str,s.str);
	return *this;
}

CString& CString::operator += (char ch) {
	int len = length();

	str = (char *) realloc(str, len + 2);
	str[len] = ch;
	str[len+1] = 0;

	return *this;
}
//------------------------------------------------------------------------------
// usefull methods
//------------------------------------------------------------------------------
void CString::clear() {
	str = (char *) realloc(str,1);
	str[0] = 0;
}

CString CString::trimRight() const {
	CString  ss(str);
	if (*ss.str) {
		char     *p = ss.str + strlen(ss.str);
		while ( (p--) != ss.str ) {
			if ( *p != ' ' ) break;
		}
		p++;
		*p = 0;
	}
	return ss;
}

CString CString::trimLeft() const {
	char     *s = str;
	while (*s == ' ') s++;
	return s;
}

CString CString::lowerCase() const {
	CString  s = str;
	int i = 0;
	while (s.str[i]) {
		s.str[i] = (char) tolower( s.str[i] );
		i++;
	}
	return s;
}

CString CString::upperCase() const {
	CString  s = str;
	int i = 0;
	while (s.str[i]) {
		s.str[i] = (char) toupper( s.str[i] );
		i++;
	}
	return s;
}

CString CString::remove(const char *pattern) const {
	CString   result;
	int       patternLength = strlen(pattern);
	if (!patternLength) return str;
	char     *temp = strdup(str);
	char     *head = temp;
	char     *tail = strstr(head,pattern);
	while (tail) {
		*tail = 0;
		result += head;
		head = tail + patternLength;
		tail = strstr(head,pattern);
	}
	result += head;
	free(temp);
	return result;
}

CString CString::replace(const char *pattern,const char *replacement) const {
	CString   result;
	int       patternLength = strlen(pattern);
	if (!patternLength) return str;
	char     *temp = strdup(str);
	char     *head = temp;
	char     *tail = strstr(head,pattern);
	while (tail) {
		*tail = 0;
		result += head;
		result += replacement;
		head = tail + patternLength;
		tail = strstr(head,pattern);
	}
	result += head;
	free(temp);
	return result;
}

CString CString::trim() const {
	CString s = trimRight();
	return s.trimLeft();
}

unsigned CString::length() const {
	return strlen(str);
}

bool CString::isEmpty() const {
	return str[0] == 0;
}

int CString::pos(const char *substr) const {
	char *p = strstr( str, substr );
	if (!p) return -1;
	return int(p - str);
}

CString  CString::capitalizeWords() const {
	CString  s = str;
	char *current = s.str;
	char *wordStart = NULL;
	if (*current) {
		for (;;) {
			if (isalnum(*current)) {
				if (!wordStart)
					wordStart = current;
			} else {
				if (current - wordStart > 3) {
					if (wordStart)
						*wordStart = toupper(*wordStart);
					else wordStart = current;
					for (char *ptr = wordStart + 1; ptr < current; ptr++)
						*ptr = tolower(*ptr);
				}
				wordStart = NULL;
			}
			if (!*current)
				break;
			current++;
		}
	}
	return s;
}

CString CString::subString(int start,int count) const {
	return CString(str+start,count);
}

int CString::toInt(int defvalue) const {
	CString t = trim();
	if (!t.length())
		return defvalue;
	int value = atoi(t.c_str());
	if (value == 0 && t != "0")
		return defvalue;
	return value;
}

double CString::toFloat(float defvalue) const {
	CString t = trim();
	if (!t.length())
		return defvalue;
	double value = atof(t.c_str());
	if (errno == ERANGE)
		return defvalue;
	return value;
}

void CString::subInsert(int start,const char *insStr) {
	int l = length();
	if (l < start) start = l;
	CString ending(str + start);
	str[start] = 0;
	*this += insStr + ending;
}

void CString::subInsert(int start,char ch) {
	char st[2] = { ch, 0 };
	subInsert(start,st);
}

void CString::subDelete(int start,int count) {
	if (count) {
		int l = strlen(str)+1;
		if (count > l) count = l;
		memmove (str+start,str+start+count,l-count-start);
	}
}

void CString::subReplace(const char *s_str,const char *r_str) {
	CString  s;
	int   l = strlen(s_str);
	char *ptext = str;
	char *pstart = strstr(str,s_str);
	while (pstart) {
		*pstart = 0;
		s += ptext;
		if (r_str)
			s += r_str;
		ptext = pstart + l;
		pstart = strstr(ptext,s_str);
	}
	s += ptext;
	assign(s.str);
}

void CString::setLength(int newLength) {
	str = (char *)realloc(str,newLength+1);
	str[newLength] = 0;
}

CString CString::wordAt(long  position) const {
	char *temp;

	CString s("",strlen(str));

   // rewind to start of current word
	while(!isspace(str[position-1]) && !ispunct(str[position-1]))
		if(position)
			position--;
		else
			break;
   // skip leading punctuation
	int len = length();
	while(ispunct(str[position]))
		if(position < len)
			position++;
		else
			break;
	temp = s.str;
	while(str[position] && !(isspace(str[position])))
	{
		if(ispunct(str[position]))
			if(str[position] != '\'')
				break;
		*temp++ = str[position++];
	}
	*temp = '\0';
	return s;
}

CString& CString::chopAt(long  limit, CString *remainder) {
	int l = length();
	if (limit > l) limit = l;
	if ( limit < 0) limit = 0;
	if (remainder)
		remainder->assign(str+limit);
	str[limit] = '\0';

	return(*this);
}

CString& CString::wrapAt(long  limit, CString *remainder) {
	if (remainder)
		*remainder = "";

	int l = length();
	if (limit > l) limit = l;
	if ( limit < 0) limit = 0;

	if (isspace(str[limit]) || (str[limit] == '\0'))
		chopAt(limit,remainder);
	else
	{
		while(limit && !isspace(str[limit]))
			limit--;
		if(limit)
			chopAt(++limit,remainder);  // skip past space, clip rest of line
	}

	return *this;
}

CString CString::substituteSpecialChars(char prefix) const {
	CString  newstr;
	char     *tempStr = strdup(str);
	char     *start = tempStr;
	char     cc[] = " ";

	for (;;) {
		char *end = strchr(start,prefix);
		if (!end) {
			newstr += start;
			break;
		}
		bool doublePrefix = false;
		if (*(end+1) == prefix) {
         // Duplicated prefix - leave one of them
			end++;
			doublePrefix = true;
		}
		*end = 0;

		newstr += start;
		start = end + 1;

		if (!doublePrefix) {
			char *nextItem;
			char charCode = (char)(int)strtol(start,&nextItem,0);
			if (nextItem != start) {
				cc[0] = charCode;
				newstr += cc;
				start = nextItem;
			}
		}
	}
	free(tempStr);
	return newstr;
}
//------------------------------------------------------------------------------
CString operator +(const char*s, const CString& rhs) {
	return CString(s) + rhs;
}
//------------------------------------------------------------------------------
bool operator <  (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) < 0;
}

bool operator <= (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) <= 0;
}

bool operator >  (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) > 0;
}

bool operator >= (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) >= 0;
}

bool operator == (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) == 0;
}

bool operator != (const CString &s1, const char *s2) {
	return strcmp(s1.str,s2) != 0;
}
//------------------------------------------------------------------------------
bool operator <  (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) < 0;
}

bool operator <= (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) <= 0;
}

bool operator >  (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) > 0;
}

bool operator >= (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) >= 0;
}

bool operator == (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) == 0;
}

bool operator != (const char *s1, const CString &s2) {
	return strcmp(s1,s2.str) != 0;
}
//------------------------------------------------------------------------------
