/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cstringlist.cpp  -  description
                             -------------------
    begin                : Tue Jan 11 2000
    copyright            : (C) 2000-2003 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <sptk/CStringList.h>
#include <sptk/CPackedStrings.h>

//---------------------------------------------------------------------------
CStringInfo::CStringInfo(const CStringInfo& si) : CString(si.str) {
	id = si.id;
}

CStringInfo::CStringInfo(const CString& s) : CString(s) {
	id = 0;
}

CStringInfo::CStringInfo(const char *s) : CString(s) {
	id = 0;
}

CStringInfo::CStringInfo() : CString("") {
	id = 0;
}

CStringInfo::CStringInfo(const char *s,int _id,int maxlen) : CString(s,maxlen) {
	id = _id;
}

CStringInfo& CStringInfo::operator =(const CStringInfo& si) {
	assign(si.str);
	id = si.id;
	return *this;
}

CStringInfo& CStringInfo::operator =(const CString& si) {
	assign(si.c_str());
	id = 0;
	return *this;
}

CStringInfo& CStringInfo::operator =(const char * s) {
	assign(s);
	id = 0;
	return *this;
}
//---------------------------------------------------------------------------
static void CharBufferToCStringList(CStringList& sl,char *buffer,const char *delimitter) {
	sl.clear();
	if (!strlen(buffer)) return;
	int dlen = strlen(delimitter);
	if (!dlen) return;
	char *p = buffer;
	for (;;) {
		char *end = strstr(p,delimitter);
		if (end) {
			int len = end - p;
         //char sc = *end;
         //*end = 0;
			sl.add(p,0,len);
         //*end = sc;
			p = end + dlen;
		} else {
			sl.add(p);
			break;
		}
	}
}
//---------------------------------------------------------------------------
CStringList::CStringList(unsigned sz) {
	m_userData = 0;
	if (sz) resize(sz);
	for (unsigned i = 0; i < sz; i++) {
		m_list[i] = new CStringInfo;
	}
}

CStringList::CStringList(const CStringList& sl) {
	m_userData = sl.m_userData;
	m_list.resize( sl.count() );
	unsigned cnt = m_list.count();
	for (unsigned i = 0; i < cnt; i++) {
		m_list[i] = new CStringInfo( sl[i] );
	}
}

CStringList::CStringList(const char *csvString,CString delimitter) {
	m_userData = 0;
	CharBufferToCStringList(*this,(char *)csvString,delimitter);
}

CStringList::CStringList(const CString& csvString,CString delimitter) {
	m_userData = 0;
	CharBufferToCStringList(*this,(char *)csvString.c_str(),delimitter.c_str());
}

CStringList::CStringList(const CBuffer& csvBuffer,CString delimitter) {
	m_userData = 0;
	CharBufferToCStringList(*this,(char *)csvBuffer.data(),delimitter.c_str());
}

CStringList::CStringList(const CPackedStrings& sl) {
	m_userData = sl.user_data();
	m_list.resize( sl.count() );
	unsigned cnt = m_list.count();
	for (unsigned i = 0; i < cnt; i++) {
		m_list[i] = new CStringInfo( sl[i] );
	}
}

CStringList::~CStringList() {
	resize(0);
}

void CStringList::fromString(const CString& csvString,const char *delimitter) {
	CharBufferToCStringList(*this,(char *)csvString.c_str(),delimitter);
}

void CStringList::fromBuffer(const CBuffer& csvBuffer,const char *delimitter) {
	CharBufferToCStringList(*this,(char *)csvBuffer.data(),delimitter);
}

void CStringList::resize(unsigned sz) {
	unsigned oldSize = m_list.count();
	if (oldSize < sz) {
      // Enlarge string list
		m_list.resize( sz );
		for (unsigned i = oldSize; i < sz; i++) {
			m_list[i] = new CStringInfo;
		}
	} else {
		for (unsigned i = sz; i < oldSize; i++) {
			CStringInfo *s = (CStringInfo *)m_list[i];
			delete s;
		}

		m_list.resize( sz );
	}
}

void CStringList::add(const CStringList& sl) {
	for (unsigned i = 0; i < sl.count(); i++)
		m_list.add(new CStringInfo(sl[i]));
}

void CStringList::insert(unsigned pos,const CStringInfo& si) {
	m_list.insert(pos,new CStringInfo(si));
}

void CStringList::insert(unsigned pos,const char *s,int id) {
	m_list.insert(pos,new CStringInfo(s,id));
}

void CStringList::insert(unsigned pos,const CStringList& sl) {
	for (int i = sl.count() - 1; i >= 0; i--)
		m_list.insert(pos,new CStringInfo(sl[i]));
}

void CStringList::remove(unsigned pos) {
	delete (CStringInfo *)m_list[pos];
	m_list.remove(pos);
}

CStringList& CStringList::operator =(const CStringList& sl) { // ASSIGNMENTOPERATOR.
	if (this == &sl) return *this;
	m_userData = sl.m_userData;
	resize( sl.count() );
	for (unsigned i = 0; i < count(); i++) {
		*(CStringInfo *)m_list[i] = *(CStringInfo *)sl.m_list[i];
	}
	return *this;
}

int CStringList::indexOf(const char *s) const {
	unsigned cnt = count();
	for (unsigned i = 0; i < cnt; i++) {
		CStringInfo *si = (CStringInfo *)m_list[i];
		if ( strcmp(si->c_str(),s) == 0 )
			return i;
	}
	return -1;
}

int CStringList::indexOf(int id) const {
	unsigned cnt = count();
	for (unsigned i = 0; i < cnt; i++) {
		CStringInfo *si = (CStringInfo *)m_list[i];
		if (si->id == id)
			return i;
	}
	return -1;
}

int sort_function(const void * s1,const void * s2) {
	const PListItem item1 = *(PListItem *)s1;
	const PListItem item2 = *(PListItem *)s2;
	const char *p1 = PCStringInfo(item1)->c_str();
	const char *p2 = PCStringInfo(item2)->c_str();
	return strcmp(p1,p2);
}

void CStringList::sort() {
	m_list.sort(sort_function);
}

CString CStringList::asString(const char *separator) const {
	CString result;
	unsigned cnt = count();
	for (unsigned i = 0; i < cnt; i++) {
		CStringInfo *si = (CStringInfo *)m_list[i];
		if (i) result += separator;
		result += si->c_str();
	}
	return result;
}

bool CStringList::saveToFile(const CString fileName) const {
	FILE *f = fopen(fileName.c_str(),"wt");
	if (!f) return false;
	for (unsigned i = 0; i < count(); i++) {
		CStringInfo *si = (CStringInfo *)m_list[i];
		fprintf(f,"%s\n",si->c_str());
	}
	fclose(f);
	return true;
}

bool CStringList::loadFromFile(const CString fileName) {
	FILE *f = fopen(fileName.c_str(),"rt");
	if (!f) return false;
	clear();
	char *buffer = new char[1024];
	while (!feof(f)) {
		if (fgets(buffer,1024,f)) {
			int l = strlen(buffer) - 1;
			while (l && (unsigned)buffer[l] < 14) {
				buffer[l] = 0;
				l--;
			}    
			add(buffer);
		}
	}
	fclose(f);
	delete buffer;
	return true;
}

CStringList& CStringList::operator += (const CStringList& list) {
	unsigned cnt = list.count();
	for (unsigned i = 0; i < cnt; i++) 
		add(list[i]);
	return *this;
}

CStringList& CStringList::operator -= (const CStringList& list) {
	unsigned lcnt = list.count();
	for (unsigned i = 0; i < lcnt; i++) {
		CStringInfo *litem = (CStringInfo *)list.m_list[i];
		int tcnt = count();
		for (int j = tcnt; j >= 0; j--) {
			CStringInfo *titem = (CStringInfo *)m_list[j];
			if (strcmp(titem->c_str(),litem->c_str()) == 0)
				remove(j);
		}
	}
	return *this;
}
