/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBaseStringMap.cpp  -  description
                             -------------------
    begin                : 13 June 2003
    copyright            : (C) 2000-2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <ctype.h>
#include <sptk/CStringMap.h>
#include <sptk/CException.h>

CBaseStringMapItem::CBaseStringMapItem(const char *name, void *data) { 
	m_name = strdup(name); 
	m_data = data; 
}

   /// Destructor.
CBaseStringMapItem::~CBaseStringMapItem() { 
	if (m_name) 
		free(m_name); 
}


CBaseStringMap::CBaseStringMap(bool ignoreCase) { 
   m_ignoreCase = ignoreCase; 
}

CBaseStringMap::~CBaseStringMap() {
	clear();
}

void CBaseStringMap::clear() { 
	unsigned cnt = m_items.count();
	for (unsigned i = 0; i < cnt; i++) {
		CBaseStringMapItem *currentItem = (CBaseStringMapItem *) m_items[i];
		deleteData(currentItem->m_data);
		delete (CBaseStringMapItem *) currentItem;
	}
	m_items.clear(); 
}

int CBaseStringMap::stringsSearch(const void *key, const void *item) {
	CBaseStringMapItem *c = *(CBaseStringMapItem **)item;
	return strcmp((char *)key,c->m_name);
}

void *CBaseStringMap::insertItem(const char *newKey,void *data) {
	if (!newKey)
		throw CException("Key can't be NULL",__FILE__,__LINE__);
	CString  key(newKey);
	if (m_ignoreCase)
		key = key.lowerCase();
	CBaseStringMapItem *currentItem = 0L;
	int compareResult = 0;
	int cnt = m_items.count();
	int pos = cnt / 2;
	int step = pos;
	if (!step) step = 1;
	if (cnt) {
		do {
			currentItem = (CBaseStringMapItem *)m_items[pos];
			compareResult = strcmp(key.c_str(),currentItem->m_name);
			if (compareResult == 0) {
            /// found existing item
				deleteData(currentItem->m_data);
				currentItem->m_data = data;
				return currentItem;
			}
			if (compareResult < 0) {
				if (step > 0)
					step /= -2;
				if (step == 0) {
					pos--;
					break;
				}
			}
			else if (compareResult > 0) {
				if (step < 0)
					step /= -2;
				if (step == 0) {
					pos++;
					break;
				}
			}
			pos += step;
			if (pos < 0) {
				step /= -2;
				pos = 0;
			}
			else
				if (pos >= cnt) {
					step /= -2;
					pos = cnt - 1;
				}
		} while (step);
	}

	if (unsigned(pos) < unsigned(cnt)) {
		currentItem = (CBaseStringMapItem *)m_items[pos];
		compareResult = strcmp(key.c_str(),currentItem->m_name);
	} else {
		currentItem = 0L;
		compareResult = 1;
	}

	CBaseStringMapItem *newItem = new CBaseStringMapItem(key.c_str(),data);

	if (compareResult == 0) { // Found existing key
		deleteData(currentItem->m_data);
		delete currentItem;
		m_items[pos] = newItem;
		return newItem;
	}

	if (pos < 0) {
		m_items.insert(0,newItem);
		return newItem;
	}

	if (pos >= cnt) {
		m_items.insert(cnt,newItem);
		return newItem;
	}

	if (compareResult < 0) {
		m_items.insert(pos,newItem);
		return newItem;
	} else {
		m_items.insert(pos+1,newItem);
	}
	return newItem;
}

void *CBaseStringMap::insert(const char *name,void *data) {
	return insertItem(name,data);
}

void *CBaseStringMap::insert(const CString& name,void *data) {
	return insertItem(name.c_str(),data);
}

void CBaseStringMap::remove(const char *name) {
	int itemIndex = findItemIndex(name);
	if (itemIndex < 0) return;
	CBaseStringMapItem **base = (CBaseStringMapItem **) m_items.base();
	CBaseStringMapItem *item = base[itemIndex];
	deleteData(item->m_data);
	delete item;
	m_items.remove(itemIndex);
}

void CBaseStringMap::remove(const CString& name) {
	remove(name.c_str());
}

int CBaseStringMap::findItemIndex(const char *str) const {
	char *key = (char *)str;
	if (m_ignoreCase) {
		key = strdup(str);
		char *p = key;
		while (*p) {
			*p = tolower(*p);
			p++;
		}
	}
	unsigned    cnt = m_items.count();
	const void *base = m_items.base();
	void *ptr = 0;
	if (cnt)
		ptr = bsearch(key,base,cnt,sizeof(void *),stringsSearch);
	if (m_ignoreCase)
		free(key);
	if (!ptr) return -1;
	return (CBaseStringMapItem **)ptr - (CBaseStringMapItem **)base;
}

CBaseStringMapItem *CBaseStringMap::findItem(const char *str) const {
   int itemIndex = findItemIndex(str);
   if (itemIndex < 0) return 0L;
	return (CBaseStringMapItem *)m_items[itemIndex];
}

void *CBaseStringMap::find(const char *str) const {
	CBaseStringMapItem *item = findItem(str);
	if (!item) return 0;
	return item->m_data;
}

void CBaseStringMap::throwNotFoundException(const char *name) const {
	throw CException("Item <"+CString(name)+"> not found.");
}
//=============================================================================
CBaseStringMapIterator::CBaseStringMapIterator(const CBaseStringMap& map) {
	m_map = &map;
	m_position = 0;
}

void *CBaseStringMapIterator::first() {
	m_position = 0;
	return operator * ();
}

void *CBaseStringMapIterator::last() {
	m_position = m_map->count() - 1;
	return operator * ();
}

void *CBaseStringMapIterator::operator *  () {
	if (m_position >= m_map->count()) 
		return 0L;
	CBaseStringMapItem *item = (CBaseStringMapItem *)m_map->m_items[m_position];
	return item->m_data;
}

void *CBaseStringMapIterator::operator ++ (int) {
	if (m_position < m_map->count())
		m_position++;
	return operator * ();
}

void *CBaseStringMapIterator::operator -- (int) {
	m_position--;
	return operator * ();
}

const char *CBaseStringMapIterator::name() const {
	if (m_position >= m_map->count()) 
		return "";
	CBaseStringMapItem *item = (CBaseStringMapItem *)m_map->m_items[m_position];
	return item->m_name;
}
