/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThread.cpp  -  description
                             -------------------
    begin                : Thu Jul 12 2001
    copyright            : (C) 2001-2005 by Alexey Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk/CThread.h>
#include <sptk/CException.h>

#include <stdio.h>
#include <string.h>

#ifndef _WIN32
void *CThread::threadStart(void *p) {
   CThread *thread = (CThread *)p;
   thread->runThread();
   return NULL;
}
#else
void CThread::threadStart(void *p) {
   CThread *thread = (CThread *)p;
   thread->runThread();
}
#endif

CThread::CThread(CString name,bool politeMode) {
   m_name = name;
   m_terminated = false;
   m_running = false;
   m_politeMode = politeMode;
}

CThread::~CThread() {
   //printf("CThread::~CThread() for %s started.\n",m_name);
   if (m_running && m_politeMode) {
      // wait till the thread is stopping
      //printf("CThread::~CThread() for %s waiting politely.\n",m_name);
      m_waiter.lock();
   }
   //printf("CThread::~CThread() for %s ended.\n",m_name);
}

void CThread::terminate()  {
   if (m_terminated) return;
   m_terminated = true;

   if (m_running && m_politeMode) {
      // wait till the thread is stopping
      m_waiter.lock();
      m_waiter.unlock();
   }
}

void CThread::createThread() {
#ifndef _WIN32
   int rc = pthread_create(&m_thread, NULL, CThread::threadStart, (void *) this);
   if (rc)
      throw CException("Can't create a new thread");
#else
   m_thread = _beginthread(CThread::threadStart, 2048, (void *) this);
   if (!m_thread)
      throw CException("Can't create a new thread");
#endif
}

void CThread::destroyThread() {
#ifndef _WIN32
   pthread_exit(NULL);
#else
   _endthread();
#endif
}

void CThread::runThread() {
   m_terminated = false;
   m_running = true;

   if (m_politeMode)
      m_waiter.lock();

   threadFunction();

   if (m_politeMode)
      m_waiter.unlock();

   destroyThread();

   m_running = false;
}

void CThread::run() {
   createThread();
}

void CThread::msleep(int msec) {
#ifndef _WIN32
   usleep(msec * 1000L);
#else
   Sleep(msec);
#endif
}
