/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               SPTK TO ODBC BRIDGE
                          CODBCChannel.cpp  -  description
                             -------------------
    begin                : Oct 6, 2003
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include "CODBCChannel.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>

double CODBCChannel::reverseByteOrder(double data) {
   char  bytes[4];
   char *src = (char *)&data;
   bytes[7] = src[0];
   bytes[6] = src[1];
   bytes[5] = src[2];
   bytes[4] = src[3];
   bytes[3] = src[4];
   bytes[2] = src[5];
   bytes[1] = src[6];
   bytes[0] = src[7];
   return *(double *)bytes;
}

CODBCChannel::CODBCChannel(CSocket *socket) {
   if (sizeof(uint32)!=4)
      throw CException("The size of type 'uint32' is not 4 bytes!");
   if (sizeof(uint16)!=2)
      throw CException("The size of type 'uint16' is not 2 bytes!");
   m_socket = socket;
   m_reverseByteOrder = (htons(12345) == 12345);
}

CODBCChannel::~CODBCChannel() {
   if (m_socket)
      m_socket->close();
}

void CODBCChannel::beginSend(CODBCCommandType command) {
   char cmd[] = "CMD ";
   m_sendBuffer.set(0L,0);
   cmd[3] = char(command);
   m_sendBuffer.append(cmd,4);
}

void CODBCChannel::commitSend() {
   m_socket->write(m_sendBuffer);
}

void CODBCChannel::sendData(int v) {
   v = htonl(v);
   m_sendBuffer.append((char *)&v,sizeof(int));
}

void CODBCChannel::sendData(double v) {
   if (m_reverseByteOrder)
      v = reverseByteOrder(v);
   m_sendBuffer.append((char *)&v,sizeof(double));
}

void CODBCChannel::sendData(const void *buffer,unsigned sz) {
   uint32 length = htonl(sz);
   m_sendBuffer.append((char *)&length,sizeof(length));
   m_sendBuffer.append((const char *)buffer,sz);
}

void CODBCChannel::readData(int& v) {
   m_socket->read((char *)&v,sizeof(int));
   v = htonl(v);
}

void CODBCChannel::readData(double& v) {
   if (m_reverseByteOrder)
      v = reverseByteOrder(v);
   m_socket->read((char *)&v,sizeof(double));
}

void CODBCChannel::readData(char *buffer,int maxsz) {
   int length;
   m_socket->read((char *)&length,sizeof(int));
   length = htonl(length);
   if (length > maxsz) {
      printf("Buffer size is %i, string in the socket is %i bytes\n",maxsz,length);
      throw CException("Buffer size is to small to receive the data.");
   }
   m_socket->read(buffer,length);
}

void CODBCChannel::readData(CString& str) {
   int length;
   int rc = m_socket->read((char *)&length,sizeof(int));
   length = htonl(length);
   str.setLength(length-1);
   rc = m_socket->read((char *)str.c_str(),length);
}

int CODBCChannel::readResponse() {
   char buffer[4];
   int len = m_socket->read(buffer,4);

   if (len < 4)
      throw CException("Expecting a command result but it isn't received.");

   if (strncmp(buffer,"CMD",3) != 0)
      throw CException("Expecting a command result but command signature isn't received.");

   CODBCCommandType command = (CODBCCommandType)buffer[3];
   if (command != OCT_RESULT) 
      throw CException("Error: Expecting a command result but command type isn't OCT_RESULT.");

   int rc;
   readData(rc);

   return rc;
}
