/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               SPTK TO ODBC BRIDGE
                          CODBCChannel.h  -  description
                             -------------------
    begin                : Oct 6, 2003
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#ifndef __CODBCCHANNEL_H__
#define __CODBCCHANNEL_H__

#include <string.h>
#include <sptk/CSocket.h>

enum CODBCCommandType { 
   OCT_NONE, OCT_RESULT, 
   OCT_DB_ERRORINFO, OCT_DB_OPEN, OCT_DB_CONNECTOPTION, OCT_DB_CLOSE,
   OCT_DB_BEGINTRAN, OCT_DB_ENDTRAN,
   OCT_QUERY_SETPARAMS, OCT_QUERY_OPEN, OCT_QUERY_EXECUTE, OCT_QUERY_CLOSE, OCT_QUERY_FREESTMT
};

typedef unsigned long  uint32;
typedef          long  int32;
typedef unsigned short uint16;
typedef          short int16;

/// Class CODBCChannel is sending and receiving ODBC data through the socket.
class CODBCChannel {
   CBuffer           m_sendBuffer;
   bool              m_reverseByteOrder;

   double reverseByteOrder(double data);

protected:

   CSocket          *m_socket;

public:
   /// Constructor
   CODBCChannel(CSocket *socket);

   /// Destructor
   ~CODBCChannel();

   /// Reset the send buffer
   void beginSend(CODBCCommandType command);

   /// Send the contents of the send buffer
   void commitSend();

   /// Appends character to the send buffer to be send with commitSend() - later
   void sendData(char ch) { m_sendBuffer.append(ch); }

   /// Appends integer data to the send buffer to be send with commitSend() - later
   void sendData(int data);

   /// Appends double data to the send buffer to be send with commitSend() - later
   void sendData(double data);

   /// Appends zero-terminated string to the send buffer to be send with commitSend() - later
   void sendData(char *buffer) { sendData(buffer,strlen(buffer)+1); }

   /// Appends buffer to the send buffer to be send with commitSend() - later
   void sendData(const void *buffer,unsigned sz);

   /// Appends string to the send buffer to be send with commitSend() - later
   void sendData(const CString& str) { sendData((char *)str.c_str(),str.length()+1); }

   /// Reads integer from the socket
   void readData(int& data);

   /// Reads double from the socket
   void readData(double& data);

   /// Reads buffer from the socket.
   void readData(char *buffer,int maxSize);

   /// Reads string from the socket
   void readData(CString& str);

   int readResponse();
};

#endif
