/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               SPTK TO ODBC BRIDGE
                          CServerODBCChannel.cpp  -  description
                             -------------------
    begin                : Jan 5, 2004
    copyright            : (C) 2003-2005 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <stdio.h>
#include <sptk/CQuery.h>
#include "CServerODBCChannel.h"

CServerODBCChannel::CServerODBCChannel(CSocket *socket)
: CODBCChannel(socket)
{
}

void CServerODBCChannel::returnError() {
	beginSend(OCT_RESULT);
	sendData(int(-1));
}

void CServerODBCChannel::returnOk() {
	beginSend(OCT_RESULT);
	sendData(int(0));
}

bool CServerODBCChannel::readCommand() {
	char buffer[4];
	if (m_socket->read(buffer,4) == 4) {
		if (strncmp(buffer,"CMD",3) != 0) {
			puts("Error: Expecting a command but command signature isn't received.");
			return false;
		}
		CODBCCommandType command = (CODBCCommandType)buffer[3];
		return readCommand(command);
	} else {
		return false;
	}
}

bool CServerODBCChannel::readCommand(CODBCCommandType command) {

	switch (command) {

		case OCT_NONE:
		case OCT_RESULT:
			puts("OCT_RESULT");
			break;

		case OCT_DB_ERRORINFO:
			puts("OCT_ERRORINFO");
			returnOk();
			sendData(m_lastError);
			commitSend();
			break;

		case OCT_DB_OPEN:
			{
				CString connectString;
				readData(connectString);
				puts("OCT_OPEN: "+CString(connectString));
				try {
					m_database.open(connectString);
					puts("  database opened.");
					returnOk();
					commitSend();
				}
				catch (CException& e) {
					m_lastError = e.text();
					returnError();
					commitSend();
				}
			}
			break;

		case OCT_DB_CLOSE:
			try {
				puts("OCT_DB_CLOSE");
				m_database.close();
				puts("  database closed.");
				returnOk();
				commitSend();
			}
			catch (CException& e) {
				m_lastError = e.text();
				returnError();
				commitSend();
			}
			catch (...) {
				m_lastError = "Unknown error in closing database";
				returnError();
				commitSend();
			}
			break;

		case OCT_DB_BEGINTRAN:
			try {
				puts("OCT_DB_BEGINTRAN");
				m_database.beginTransaction();
				returnOk();
				commitSend();
			}
			catch (CException& e) {
				m_lastError = e.text();
				returnError();
				commitSend();
			}
			catch (...) {
				m_lastError = "Unknown error in starting transaction";
				returnError();
				commitSend();
			}
			break;

		case OCT_DB_ENDTRAN:
			try {
				puts("OCT_DB_ENDTRAN");
				int commit;
				readData(commit);
				m_database.endTransaction(commit);
				returnOk();
				commitSend();
			}
			catch (CException& e) {
				m_lastError = e.text();
				returnError();
				commitSend();
			}
			catch (...) {
				m_lastError = "Unknown error in ending transaction";
				returnError();
				commitSend();
			}
			break;

		case OCT_QUERY_OPEN:
			{
				puts("OCT_QUERY_OPEN");
				try {
					int queryHandle;
					CString querySQL;
					readData(queryHandle);
					readData(querySQL);
					puts("  "+querySQL);
					CQuery *query = (CQuery *)queryHandle;
					if (!query) {
						query = new CQuery(&m_database,querySQL);
					} else {
						query->sql(querySQL);
					}
					query->open();
					returnOk();
					sendData((int)query);
					commitSend();
				}
				catch (CException& e) {
					m_lastError = e.text();
					returnError();
					commitSend();
				}
			}
			break;

		case OCT_QUERY_CLOSE:
			{
				puts("OCT_QUERY_CLOSE");
				try {
					int queryHandle;
					readData(queryHandle);
					CQuery *query = (CQuery *)queryHandle;
					if (query)
						query->close();
					returnOk();
					commitSend();
				}
				catch (CException& e) {
					m_lastError = e.text();
					returnError();
					commitSend();
				}
			}
			break;

		case OCT_QUERY_SETPARAMS:
			{
				puts("OCT_QUERY_SETPARAMS");
				try {
					int queryHandle;
					int paramCount;
					readData(queryHandle);
					readData(paramCount);
					CQuery *query = (CQuery *)queryHandle;
					for (int i = 0; i < paramCount; i++) {
					}
					returnOk();
					sendData((int)query);
					commitSend();
				}
				catch (CException& e) {
					m_lastError = e.text();
					returnError();
					commitSend();
				}
			}
			break;

		case OCT_QUERY_FREESTMT:
			{
				puts("OCT_QUERY_FREESTMT");
				try {
					CString sql;
					int     handle;
					readData(handle);
					m_database.queryFreeStmt((CQuery *)handle);
					returnOk();
					commitSend();
				}
				catch (CException& e) {
					m_lastError = e.text();
					returnError();
					commitSend();
				}
			}
			break;
//       case OCT_EXECUTE:
//          puts("OCT_EXECUTE");
//          {
//             CString stmtText;
//             readData(statementHandle);
//             rc = SQLExecute(statementHandle);
//             if (Successful(rc)) {
//                returnOk();
//             } else {
//                returnError();
//             }
//             commitSend();
//          }
//          break;
//          
//       case OCT_SETCONNECTOPTION:
//          puts("OCT_SETCONNECTOPTION");
//          {
//             int option;
//             int value;
//             //readData(m_connectionHandle);
//             readData(option);
//             readData(value);
//             //rc = SQLSetConnectOption(m_connectionHandle,(SQLUSMALLINT)option,(SQLULEN)value);
//             if (Successful(rc)) {
//                returnOk();
//             } else {
//                returnError();
//             }
//             commitSend();
//          }
//          break;

		default:
			puts("UNKNOWN COMMAND");
			break;
	}
	return true;
}
