/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlDoc.cpp  -  description
                             -------------------
    begin                : Sun May 22 2003
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2003 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <ctype.h>
#include <stdlib.h>

#include <sptk/xml/CXmlDoc.h>
#include <sptk/xml/CXmlParser.h>

int CXmlDoc::m_indent_spaces = 2;

//////////////////////////

CXmlDoc::CXmlDoc()
   : CXmlNode("#document", DOM_DOCUMENT, 0)
{
   m_parent = 0;     
}

CXmlNode *CXmlDoc::rootNode()
{
   for(unsigned n=0; n < children(); n++) {
      CXmlNode *nd = child(n);
      if(nd->type()==DOM_ELEMENT)
         return nd;
   }
   return 0;
}

void CXmlDoc::clear()
{
   CXmlNode::clear();
}

CXmlNode *CXmlDoc::createElement(const char *tagname)
{
   CXmlNode *node = new CXmlNode(tagname, DOM_ELEMENT, this);
   return node;
}

CXmlNode *CXmlDoc::createText(const char *data)
{
   CXmlNode *node = new CXmlNode("#text", DOM_TEXT, this);
   node->value(data);
   return node;
}

CXmlNode *CXmlDoc::createComment(const char *data)
{
   CXmlNode *node = new CXmlNode("#comment", DOM_COMMENT, this);
   node->value(data);
   return node;
}

CXmlNode *CXmlDoc::createCDataSection(const char *data)
{
   CXmlNode *node = new CXmlNode("#cdata-section", DOM_CDATA_SECTION, this);
   node->value(data);
   return node;
}

CXmlNode *CXmlDoc::createPI(const char *target, const char *data)
{
   CXmlNode *node = new CXmlNode(target, DOM_PI, this);
   node->value(data);
   return node;
}

//#define WRITE(text) tmp.clear(); tmp=(text); buffer.append(tmp)

void CXmlDoc::save(CBuffer &buffer,int /*indent*/) const {
   CString tmp;
   unsigned n;
   CXmlNode *xml_pi = 0;

   // Write XML PI
   for (n=0; n<children(); n++) {
      if(child(n)->isPI() && child(n)->name().upperCase()=="XML") {
         xml_pi = child(n);
         xml_pi->save(buffer);
      }
   }

   if (!docType().name().isEmpty()) {
      buffer.append("<!DOCTYPE " + docType().name());
      if(!docType().systemID().isEmpty()) {
         buffer.append(" SYSTEM \"" + docType().systemID() + "\"");

         if(!docType().publicID().isEmpty()) {
            buffer.append(" PUBLIC \"" + docType().publicID() + "\"");
         }
      }
      if(docType().entities().count() > 0) {
         buffer.append(" [\n");
         CXmlEntities::Iterator it(docType().entities());
         for(; *it; it++) {
            buffer.append("<!ENTITY " + CString(it.name()) + " \"" + *(*it) + "\">\n");
         }
         buffer.append("]");
      }
      buffer.append(">\n");
   }

    // call save() method of the first (and hopefully only) node in xmldocument
   for(n=0; n<children(); n++) {
      CXmlNode *nd = child(n);
      if(nd == xml_pi) continue;
      nd->save(buffer, 0);
   }
}

//////////////////////////////////////

CXmlDocType::CXmlDocType(const char *name, const char *public_id, const char *system_id)
{
   m_html = false;
   m_name = name;
   if(public_id) m_public_id = public_id;
   if(system_id) m_system_id = system_id;
}

struct entity {
   const char *name;
   int replacement_len;
   const char *replacement;
};

struct entity builtin_ent_html[] = {
#include "html_entities.h"
};
int html_ent_size = (int)(sizeof(builtin_ent_html)/sizeof(builtin_ent_html[0]));

class HTML_EntityCache
{
   CStringMap<struct entity *> hash;
public:
   HTML_EntityCache() : hash() {
      for(int n=0; n<html_ent_size; n++) {
         struct entity *ent = &builtin_ent_html[n];
         hash.insert(ent->name, ent);
      }
   }
   const struct entity *find(const CString &ent) const {
      return hash[ent];
   }
};
static const HTML_EntityCache html_entities;

struct entity builtin_ent_xml[] =
{
   { "amp", 1, "&" },
   { "lt",  1, "<" },
   { "gt",  1, ">" },
   { "apos",  1, "'" },
   { "quot",  1, "\"" }
};
int xml_ent_size = (int)(sizeof(builtin_ent_xml)/sizeof(builtin_ent_xml[0]));

class XML_EntityCache
{
   CStringMap<struct entity *> hash;
public:
   XML_EntityCache() : hash() {
      for(int n=0; n<xml_ent_size; n++) {
         struct entity *ent = &builtin_ent_xml[n];
         hash.insert(ent->name, ent);
      }
   }
   const struct entity *find(const CString &ent) const {
      struct entity **ptr = hash.find(ent);
      if (ptr) return *ptr;
      return NULL;
   }
};
static const XML_EntityCache xml_entities;


bool CXmlDocType::decodeEntities(const char *str, CString &ret)
{
    //Fast check..
   if(!strchr(str, '&')) return false;

   int str_len = strlen(str);
   bool replaced = false;

   for(int n=0; n<str_len; n++) 
   {
      if(str[n]=='&') {
         int end=n+1;
         for(; end < str_len; end++) {
            if(str[end]==';')
               break;
         }
         CString ent(str+n+1, end-(n+1));
         CString rep = getReplacement(ent);
         if(rep!=ent) {
            n += ent.length()+1;
            ret += rep;
            replaced = true;
         } else
            ret += str[n];
      } else
         ret += str[n];
   }

   return replaced;
}

bool CXmlDocType::encodeEntities(const char *str, CString &ret) {
   int num;
   struct entity *table;
   if(m_html) {
      num = html_ent_size;
      table = builtin_ent_html;
   } else {
      num = xml_ent_size;
      table = builtin_ent_xml;
   }

   bool replaced = false;

   const char *ptr = str;
   while(*ptr) {
      bool found=false;
      for (int i = 0; i < num; i++) {
         if(!strncmp(ptr, table[i].replacement, table[i].replacement_len)) {
            ret += '&';
            ret += table[i].name;
            ret += ';';
            found = replaced = true;
         }
      }

      if(!found) {
         CXmlEntities::Iterator it(m_entities);
         for(; *it; it++) {
            CString& val = it.value();
            unsigned len = val.length();
            if(!strncmp(ptr, val.c_str(), len)) {
               ret += '&';
               ret += it.name();
               ret += ';';
               found = replaced = true;
               ptr += len-1;
            }
         }
      }

      if(!found)
         ret += *ptr;

      ptr++;
   }

   return replaced;
}

CString CXmlDocType::getReplacement(const char *name) const {
   // &#123; style entity..
   if(name[0]=='#') {
      char tmp[6]; 
      if(isdigit(name[1])) {
         tmp[0] = (char)strtol(name+1, NULL, 10);
         tmp[1] = '\0';
         return CString(tmp,2);
      } else if(name[1]=='x' || name[1]=='X') {
         tmp[0] = (char)strtol(name+2, NULL, 16);
         tmp[1] = '\0';
         return CString(tmp,2);
      }
   }

   // Find in built-ins, see entities.h
   const struct entity *entity;
   if (m_html)
      entity = html_entities.find(name);
   else
      entity = xml_entities.find(name);
   if (entity) return CString(entity->replacement, entity->replacement_len);

   // Find in custom attributes
   if (m_entities.contains(name))
      return m_entities[name];

   return name;
}

bool CXmlDocType::hasEntity(const char *name) const {
   CString tmp = getReplacement(name);
   return (tmp!=name);
}
